(function () {

    var $container = AJS.CRU.DIALOG.$CONTAINER;

    var dialogHandler = function (callbacks) {
        return function (event) {
            var dialog = $container.data('dialog');
            var permaId = $container.data('permaId');

            if (AJS.CRU.UTIL.isReviewPage() && callbacks.review) {
                return callbacks.review(dialog, permaId, event);
            } else if (!AJS.CRU.UTIL.isReviewPage() && callbacks.external) {
                return callbacks.external(dialog, permaId, event);
            }
        };
    };

    var viewFiltersHandler = function (filters) {
        return dialogHandler({
            review: function (dialog) {
                if (AJS.$('body').hasClass('review-updated')) {
                    window.location.hash = 'f-' + filters.join(',');
                    window.location.reload(true);
                } else {
                    AJS.CRU.REVIEW.UTIL.filterAndExpandFrxs(filters);
                    dialog.remove();
                }
            },
            external: function (dialog, permaId) {
                window.location = AJS.CRU.UTIL.urlBase(permaId) + '#f-' + filters.join(',');
            }
        });
    };

    var batchProcessDraftComments = function (action, permaId, onComplete) {
        var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/draftCommentsAjax';
        var params = {
            action: action
        };
        var done = function (resp) {
            if (onComplete) {
                onComplete(resp);
            }
        };
        AJS.FECRU.AJAX.ajaxDo(url, params, done, true);
    };

    var draftCommentsHandler = function (action) {
        return dialogHandler({
            review:  function (dialog) {
                AJS.$.each(review.draftComments(), function (i, comment) {
                    commentator[action + 'Comment'](comment.id(), review.id());
                });
                AJS.$('#dialog-drafts-links').hide();
            },
            external: function (dialog, permaId) {
                batchProcessDraftComments(action, permaId, function (resp) {
                    if (resp.worked) {
                        AJS.$('#dialog-drafts-links').hide();
                    } else {
                        // TODO Show error message within dialog panel.
                    }
                });
            }
        });
    };

    AJS.$(document).ready(function () {
        AJS.$('#dialog-view-drafts').live('click', viewFiltersHandler(['draftcomments']));
        AJS.$('#dialog-view-unread-comments').live('click', viewFiltersHandler(['unreadcomments']));
        AJS.$('#dialog-view-incomplete-frxs').live('click', viewFiltersHandler(['incomplete']));
        AJS.$('#dialog-view-unresolved-jiras').live('click', viewFiltersHandler(['unresolvedsubtasks']));

        AJS.$('#dialog-delete-drafts').live('click', draftCommentsHandler('delete'));
        AJS.$('#dialog-post-drafts').live('click', draftCommentsHandler('publish'));

        AJS.$('.dialog-add-to-review').live('click', function () {
            var permaId = AJS.$(this).attr('id').replace(/^dialog-add-to-review-/, '');
            var $container = AJS.CRU.DIALOG.$CONTAINER;
            $container.data('dialog').remove();
            AJS.CRU.UTIL.addToReview(AJS.$.extend($container.data('params'), {
                task: permaId
            }));
        });
    });

})();
