if (!AJS.CRU) {
    AJS.CRU = {};
}
AJS.CRU.COMMENT = {};

var commentAjaxController = (function() {
    /*****************
     * PRIVATE STUFF *
     *****************/
    function isEmptyComment(form, nameprefix) {
        var empty = false;
        var newText = form['newText'];

        if (newText && /^\s*$/.test(newText.value)) {
            empty = true;
            AJS.$("#"+nameprefix + "AutosaveMsg").html("Cannot save or post an empty comment.");
        }

        return empty;
    }

    function progressIssueLinkClass(commentId, from, to) {
        getCommentElements(commentId, 'commentIssueKey_')
                .removeClass('commentIssueKey'+from)
                .addClass('commentIssueKey'+to);
        getCommentElements(commentId, 'commentIssueStatus_')
                .removeClass('commentIssueStatus'+from)
                .addClass('commentIssueStatus'+to);
        if (to == "Linking") {
            getCommentElements(commentId, 'busyCommentIssue').show();
        } else {
            getCommentElements(commentId, 'busyCommentIssue').hide();
        }
    }

    function getCommentElements(commentId, elementSuffix) {
        var comment = review.comment(commentId);
        if (comment.isInline()) {
            return AJS.$('#above' + elementSuffix + commentId +
                         ',#inline' + elementSuffix + commentId);
        } else if (comment.frx()) {
            return AJS.$('#revision' + elementSuffix + commentId);
        } else {
            return AJS.$('#general' + elementSuffix + commentId);
        }
    }

    function commentButtonSyncWrapper(callback) {
        return function (resp) {
            if (callback) {
                callback(resp);
            }
            commentator.syncCommentButtons();
        };
    }

    /*****************
     * PUBLIC  STUFF *
     *****************/
    var res = {
        // General comment methods
        publishComment : function (commentId, permaId, done) {
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/publishCommentAjax/';
            var params = { "commentId" : commentId };

            var onDone = function(resp) {
                if(done) {
                    done(commentId, resp);
                }
            };
            var onComplete = commentButtonSyncWrapper(onDone);
            AJS.FECRU.AJAX.ajaxDo(url, params, onComplete);
            return false;
        },

        addReply : function (permaId, form, draft) {
            var $form = AJS.$(form);
            var $decal = $form.children("fieldset").children(".decal");
            if (!isEmptyComment(form,"reply")) {
                $decal.addClass("spinner");
                commentator.commentLock = true;
                var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/replyAjax/';
                var params = $form.serialize() + "&draft=" + (draft ? draft : false);
                var done = function(resp) {
                    $decal.removeClass("spinner");
                    commentator.insertAjaxReply(resp); //checks worked
                    commentator.updateCommentCount(resp);
                    commentator.commentLock = false;
                };
                var onComplete = commentButtonSyncWrapper(done);
                AJS.FECRU.AJAX.ajaxDo(url, params, onComplete);
            }
            return false;
        },

        addGeneralComment : function (permaId, form, draft) {
            var $form = AJS.$(form);
            var $decal = $form.children("fieldset").children(".decal");
            if (!isEmptyComment(form,"general")) {
                $decal.addClass("spinner");
                commentator.commentLock = true;
                var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/generalCommentAjax/';
                var params = $form.serialize() + "&draft=" + (draft ? draft : false);
                var done = function(resp) {
                    $decal.removeClass("spinner");
                    commentator.insertAjaxGeneralComment(resp); //checks worked
                    commentator.commentLock = false;
                };
                var onComplete = commentButtonSyncWrapper(done);
                AJS.FECRU.AJAX.ajaxDo(url, params, onComplete);
            }
            return false;
        },

        addRevisionComment : function (permaId, form, draft, namePrefix) {
            var $form = AJS.$(form);
            var $decal = $form.children("fieldset").children(".decal");
            if (!isEmptyComment(form, namePrefix)) {
                commentator.commentLock = true;
                $decal.addClass("spinner");
                var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/revisionCommentAjax/';
                var params = $form.serialize() + "&draft=" + (draft ? draft : false);
                var done = function(resp) {
                    $decal.removeClass("spinner");
                    commentator.insertRevisionComment(resp); //checks worked
                    if (resp.worked) {
                        tetrisCommentController.renderTetrisCommentMarkersForComment(resp.comment.id);
                    }
                    commentator.commentLock = false;
                };
                var onComplete = commentButtonSyncWrapper(done);
                AJS.FECRU.AJAX.ajaxDo(url, params, onComplete);
            }
            return false;
        },

        autoSaveReply : function (permaId, form, done) {
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/replyAjax/';
            var params = AJS.$(form).serialize() + "&draft=true";
            AJS.FECRU.AJAX.ajaxDo(url, params, done, true);
        },

        autoSaveGeneralComment : function (permaId, form, done) {
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/generalCommentAjax/';
            var params = AJS.$(form).serialize() + "&draft=true";
            AJS.FECRU.AJAX.ajaxDo(url, params, done, true);
        },

        autoSaveRevisionComment : function (permaId, form, done) {
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/revisionCommentAjax/';
            var params = AJS.$(form).serialize() + "&draft=true";
            AJS.FECRU.AJAX.ajaxDo(url, params, done, true);
        },

        deleteComment : function (commentId, permaId, done) {
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/deleteCommentAjax/';
            var params = { "commentId" : commentId };
            var onComplete = commentButtonSyncWrapper(done);
            AJS.FECRU.AJAX.ajaxDo(url, params, onComplete);
            return false;
        },

        // Comment read status
        updateCommentReadStatus : function(commentId, permaId, markAsRead, done) {
            // Prevent simultaneous requests.

            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/commentReadStatusAjax/';
            var params = { "commentId" : commentId, "markAsRead" : markAsRead };

            var onComplete = commentButtonSyncWrapper(done);
            AJS.FECRU.AJAX.ajaxDo(url, params, onComplete);
            return false;
        },

        markAllCommentsRead : function (updatedComments, permaId, done) {
            if (updatedComments.length > 0) {
                var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/commentReadStatusAjax/';
                var params = {
                    markAsRead: true,
                    cid: AJS.$.map(updatedComments, function (comment) { return comment.id(); })
                };

                var onComplete = commentButtonSyncWrapper(done);
                AJS.FECRU.AJAX.ajaxDo(url, params, onComplete);
            }
            return false;
        },

        loadCommentIssueStatuses : function (frxId) {
            var commentIdsWithLinkedIssues = [];
            var comments;
            if (frxId) {
                comments = review.frx(frxId).comments();
            } else {
                comments = review.generalComments();
            }
            AJS.$.each(comments, function() {
                var comment = review.comment(this.id());
                if (comment.issueKey() && comment.issueKey() != "") {
                    commentIdsWithLinkedIssues.push(comment.id());
                }
            });
            if (commentIdsWithLinkedIssues.length === 0) {
                return;
            }
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/getCommentIssueStatusAjax/';
            var params = { "commentIds" : commentIdsWithLinkedIssues };
            var done = function(resp) {
                if (resp.worked) {
                    var issues = resp.issues;
                    AJS.$.each(issues, function() {
                        var commentId = this.commentId;
                        var $statusElems = getCommentElements(commentId, 'commentIssueStatus_');
                        $statusElems.text(this.issueStatus);
                        review.comment(commentId).setIssueStatus(this.issueStatus);
                        var statusElemText = $statusElems.filter(':first').text().toLowerCase();
                        if (statusElemText === 'resolved' || statusElemText === 'closed' || statusElemText === 'error') {
                            getCommentElements(commentId, 'commentIssueResolve_').hide();
                        } else {
                            getCommentElements(commentId, 'commentIssueResolve_').show();
                        }
                    });
                }
            };
            AJS.FECRU.AJAX.ajaxDo(url, params, done);
        },

        resolveIssue : function (commentId) {
            getCommentElements(commentId, 'busyCommentIssue_').show();
            getCommentElements(commentId, 'commentIssueResolve_').hide();
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/resolveCommentIssueAjax/';
            var params = { "commentId" : commentId };
            var done = function(resp) {
                var $statusElems = getCommentElements(commentId, 'commentIssueStatus_');
                if (resp.worked) {
                    $statusElems.text(resp.issueStatus);
                    review.comment(commentId).setIssueStatus(resp.issueStatus);
                }
                var $statusElem = $statusElems.filter(':first');
                if ($statusElem.text() == 'Resolved' || $statusElem.text() == 'Error') {
                    getCommentElements(commentId, 'commentIssueResolve_').hide();
                } else {
                    getCommentElements(commentId, 'commentIssueResolve_').show();
                }
                getCommentElements(commentId, 'busyCommentIssue_').hide();
            };
            AJS.FECRU.AJAX.ajaxDo(url, params, done);
        },

        linkIssue : function (commentId) {
            getCommentElements(commentId, 'busyCommentIssue_').show();
            progressIssueLinkClass(commentId, 'Unlinked', 'Linking');
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/linkCommentIssueAjax/';
            var params = { "commentId" : commentId };
            var done = function(resp) {
                if (resp.worked) {
                    var comment = review.comment(commentId);
                    getCommentElements(commentId, 'commentIssueKey_').html(resp.issueHoverHtml);
                    comment.setIssueKey(resp.issueKey);
                    comment.setIssueStatus(resp.issueStatus);
                    progressIssueLinkClass(commentId, 'Linking', 'Linked');

                    var $statusElems = getCommentElements(commentId, 'commentIssueStatus_');
                    $statusElems.text(resp.issueStatus);
                    if ($statusElems.text().indexOf('Resolved') == -1 && $statusElems.text().indexOf('Error') == -1) {
                        getCommentElements(commentId, 'commentIssueResolve_').show();
                    } else {
                        getCommentElements(commentId, 'commentIssueResolve_').hide();
                    }

                } else {
                    progressIssueLinkClass(commentId, 'Linking', 'Unlinked');
                }
                getCommentElements(commentId, 'busyCommentIssue_').hide();
            };
            AJS.FECRU.AJAX.ajaxDo(url, params, done);
        }

    };
    return res;
})();
