AJS.CRU.COMMENT.FORMS = {};

(function() {

    /**
     * CommentForm class
     * Base commentForm class extend for specific forms
     */
    var CommentForm = function (id, source) {
        var name = id;
        var $form;
        var $textBox;
        var inited = false;
        var sourceForm = source;

        this.con = function(id, source) {
            name = id;
            sourceForm = source;
        };

        this.init = function() {
            if(!inited){
                var $formSource = AJS.$("#"+sourceForm);
                if ($formSource.length === 1) {
                    $form = $formSource.clone();
                    $form.attr('id',name)
                         .attr('name',name);
                    // setAttribute does not work for name in IE! so manually replace the name
                    $form.html( $form.html().replace(/source/,name) )
                         .html( $form.html().replace(/DefectFields/g,'DF') );
                    inited = true;
                }
            }
        };

        this.getForm = function() {
            this.init();
            return $form;
        };

        this.getFormElement = function(name) {
            this.init();
            return $form.find('[name='+name+']');
        };

        this.getTextBox = function() {
            if( !$textBox || $textBox.length !== 1) {
                $textBox = this.getFormElement('newText');
            }
            return $textBox;
        };

        this.setTextChangeListener = function(l) {
            this.getTextBox().bind("onkeyup", l );
        };

        this.getCommentText = function() {
            return this.getTextBox().val();
        };

        this.setDefaultFocus = function() {
            (this.getTextBox()[0]).focus();
        };

        this.clearForm = function() {
            this.resetButtons();
            var clear = function() {
                try {
                    var $element = AJS.$(this);
                    var type = $element.attr("type");
                    if(type == "checkbox") {
                        $element.removeAttr("checked");
                        $element.val(false);
                    } else if (type == "button" || type == "submit") {
                        // don't do anything to button's values
                        // (in IE6/7, clearing .value clears the button text)
                        $element.removeAttr("disabled"); // but make sure they are enabled
                    } else {
                        $element.val("");
                    }
                } catch(e) {
                    //ignore. happens in safari/konqueror when comment form is cleared when not displayed
                }
            };

            // this.getForm().find('input, select, textarea') fails under
            // Safari 3.2 and jQuery 1.3.2 when the form is not yet in the DOM.
            // See http://dev.jquery.com/ticket/4554
            var $form = this.getForm();
            $form.find("input").each(clear);
            $form.find("select").each(clear);
            $form.find("textarea").each(clear);
        };

        this.resetButtons = function() {
            this.toggleButtons(true);
            this.getFormElement("saveAsDraft").text("Save as Draft");
            this.getFormElement("cancelComment").show();
            this.getFormElement("discardComment").hide();
        };

        this.activateAutosaveButtons = function() {
            this.getFormElement("saveAsDraft").text("Keep as Draft");
            this.getFormElement("cancelComment").hide();
            this.getFormElement("discardComment").show();
        };

        this.toggleButtons = function(enable) {
            this.getForm().children("button").each( function() {
                try{
                    this.disabled = !enable;
                } catch(e) {
                    //ignore. happens in safari/konqueror when comment form is cleared when not displayed
                }
            } );
        };

        this.hasCommentId = function() {
            var cid = this.getCommentId();
            return cid != "" && cid != "-1";
        };

        this.getCommentId = function () {
            return this.getFormElement("commentId").val();
        };

        this.setFormVal = function(name, val) {
            this.init();
            try{
                var $formEl = this.getFormElement(name);
                if( $formEl.attr("type") == "checkbox") {
                    $formEl.attr("checked", val ? "checked" : "");
                }
                $formEl.val(val);
            } catch(e) {
                //ignore. happens in safari/konqueror when comment form is cleared when not displayed
            }
        };

        this.setFormElementVis = function(name, val) {
            this.init();
            try {
                this.getFormElement(name).css("display", val);
            } catch(e) {
                //ignore. happens in safari/konqueror when comment form is cleared when not displayed
            }
        };

        this.showMetrics = function(prefix, show) {
            var nodeName = prefix + "DF";
            toggleNodeAndImage(nodeName, show, !show, true);
        };
    };

    /**
     * CommentForm class
     * This holds a single revision comment form and controls access to the form as well as providing
     * useful methods
     */
    var RevisionCommentForm = function (n, s) {
        this.con(n,s);

        this.setFromLines = function(fromLines) {
            this.setFormVal("fromLineRange",fromLines);
        };

        this.setToLines = function(toLines) {
            this.setFormVal("toLineRange",toLines);
        };

        this.setFrxId = function(id) {
            this.setFormVal("frxId",id);
        };

        this.setFromFrxRevision = function(id) {
            this.setFormVal("fromFrxRevisionId",id);
        };

        this.setToFrxRevision = function(id) {
            this.setFormVal("toFrxRevisionId",id);
        };

        this.clearComment = function() {
            this.clearForm();
            this.showMetrics("revision", false);
            this.setFormElementVis("saveAsDraft", "");
        };

        this.populateFromComment = function(comment) {
            this.setFormVal("newText",comment.message());
            this.setFormVal("commentId",comment.id());
            this.setFormVal("toLineRange",comment.toLineRange());
            this.setFormVal("fromLineRange",comment.fromLineRange());
            this.setFormVal("defect",comment.defect());
            var rcf = this;
            AJS.$.each(comment.metrics(), function() {
                rcf.setFormVal(this.label, this.value);
            });
            this.showMetrics("revision", comment.defect());
            if (!comment.draft()) {
                this.setFormElementVis("saveAsDraft", "none");
            }
        };
    };
    RevisionCommentForm.prototype = new CommentForm();

    AJS.CRU.COMMENT.FORMS.RevisionCommentForm = RevisionCommentForm;


    var FileCommentForm = function (n, s) {
        this.con(n,s);

        this.setFrxId = function(id) {
            this.setFormVal("frxId",id);
        };

        this.clearComment = function() {
            this.clearForm();
            this.showMetrics("file", false);
            this.setFormElementVis("saveAsDraft", "");
        };

        this.populateFromComment = function(comment) {
            this.setFormVal("newText",comment.message());
            this.setFormVal("commentId",comment.id());
            this.setFormVal("defect",comment.defect());
            var rcf = this;
            AJS.$.each(comment.metrics(), function() {
                rcf.setFormVal(this.label, this.value);
            });
            this.showMetrics("file", comment.defect());
            if (!comment.draft()) {
                this.setFormElementVis("saveAsDraft", "none");
            }
        };
    };
    FileCommentForm.prototype = new CommentForm();

    AJS.CRU.COMMENT.FORMS.FileCommentForm = FileCommentForm;

    var GeneralCommentForm = function (n, s) {
        this.con(n,s);
        this.clearComment = function(){
            this.clearForm();
            this.showMetrics("general", false);
            this.setFormElementVis("saveAsDraft", "");
        };

        this.populateFromComment = function(comment){
            this.setFormVal("newText",comment.message());
            this.setFormVal("commentId",comment.id());
            this.setFormVal("defect",comment.defect());
            var gcf = this;
            AJS.$.each(comment.metrics(), function() {
                gcf.setFormVal(this.label, this.value);
            });
            this.showMetrics("general", comment.defect());
            if (!comment.draft()) {
                this.setFormElementVis("saveAsDraft", "none");
            }
        };
    };
    GeneralCommentForm.prototype = new CommentForm();

    AJS.CRU.COMMENT.FORMS.GeneralCommentForm = GeneralCommentForm;

    var ReplyCommentForm = function (n, s) {
        this.con(n,s);

        this.clearComment = function(){
            this.clearForm();
            this.setFormElementVis("saveAsDraft", "");
        };

        this.setReplyToId = function(replyToId){
            this.setFormVal("replyToId", replyToId);
        };

        this.populateFromComment = function(comment){
            this.setFormVal("newText",comment.message());
            this.setFormVal("commentId",comment.id());
            if (!comment.draft()) {
                this.setFormElementVis("saveAsDraft", "none");
            }
        };
    };
    ReplyCommentForm.prototype = new CommentForm();

    AJS.CRU.COMMENT.FORMS.ReplyCommentForm = ReplyCommentForm;

    /**
     * This is a CommentForm handler class that looks after inserting/removing a comment form inside a parent element
     * It also manages other elements you wish to hide or unhide while displaying the element
     */
    var CommentFormWrangler = function (commentform, namePrefix, autosaveFuncName) {
        var that = this;
        var $switchElement;
        var cForm = commentform;
        var $insertParent;
        var $insertPoint;
        var insertRow = false;
        var displaying = false;
        var $wrapperDiv;
        var wrapperRow;
        var divWidth;
        var inited = false;
        var changeParentDisplay = false;
        var autosaveChecksEnabled = false;
        var lastAutosaveDate = new Date();
        var lastTextChangeDate = new Date();
        var lastAutosaveText = "";

        var TableRow = function (id) {
            var $cell = AJS.$(document.createElement('td'));
            var $row  = AJS.$(document.createElement('tr'))
                            .attr('id',id)
                            .append($cell);

            this.setSpan = function(colSpan){
                $cell.attr('colSpan',colSpan);
            };

            this.getCell = function() {
                return $cell;
            };

            this.getRow = function() {
                return $row;
            };
        };

        function init() {
            if(!inited) {
                $wrapperDiv = AJS.$(document.createElement('div'));
                wrapperRow = new TableRow('commentTR');
                $wrapperDiv.addClass('commentForm');
                $wrapperDiv.append(cForm.getForm());

                cForm.setTextChangeListener(that.autosaveTextChanged);

                inited = true;
            }
        }

        this.getCommentForm = function(){
            init();
            return cForm;
        };

        this.setWidth = function(width){
            $wrapperDiv.width(width);
            divWidth = width;
        };

        this.unsetWidth = function() {
            $wrapperDiv.width('');
            divWidth = null;
        };

        this.setInsertPoint = function(parent, insertAfter) {
            init();
            $insertParent = AJS.$(parent);
            $insertPoint = AJS.$(insertAfter);
            insertRow = $insertParent.is('TABLE') || ($insertParent.is('TBODY') );
        };

        this.setSwitchElement = function(elementId){
            init();
            $switchElement = AJS.$("#"+elementId);
        };

        this.isDisplaying = function(){
            init();
            return displaying;
        };

        this.getParent = function() {
            return $insertParent;
        };

        this.isDisplayingRow = function(){
            init();
            return insertRow;
        };

        function switchParentOn(){
            if($insertParent.is(":hidden")) {
                changeParentDisplay = true;
                $insertParent.show();
            } else {
                changeParentDisplay = false;
            }
            $switchElement.hide();
        }

        function switchParentOff(){
            if(changeParentDisplay) {
                $insertParent.hide();
            }
            changeParentDisplay = false;
            $switchElement.show();
        }

        /*
         insert/append the element to the parent and hide all the switch elements
        */
        this.insertAndSwitch = function(){
            init();
            if(!displaying) {
                if(insertRow) { //if Row the insertpoint MUST be a ROW
                    // HACK! any colspan which is larger than the actual colspan makes the row fit nice and snug.
                    // We could easily calculate the true colspan of a table row (tr[cols].length is incorrect though)
                    // but for performance reasons it's just as easy to let the browser do the work. Fuck browsers.
                    wrapperRow.setSpan('99');
                    wrapperRow.getCell().append($wrapperDiv);
                    wrapperRow.getRow().show().insertAfter($insertPoint);
                } else {
                    $wrapperDiv.appendTo($insertParent);
                }
                $wrapperDiv.show();
                switchParentOn();
                displaying = true;
            }
        };

        this.removeAndSwitch = function(){
            init();
            if(displaying) {
                if(insertRow) {
                    $wrapperDiv.hide();
                    wrapperRow.getRow().hide();
                } else {
                    $wrapperDiv.hide();
                }
                switchParentOff();
                displaying = false;
            }
        };

        /* remove the element from current insertpoint restoring all current switch elements and
           put it in the new insert point with the new switch and display
        */
        this.exchange = function(parent, insertBefore, newSwitchElementId){
            init();
            if(displaying){
                this.removeAndSwitch();
            }
            this.setSwitchElement(newSwitchElementId);
            this.setInsertPoint(parent, insertBefore);
            this.insertAndSwitch();
        };

        this.move = function (insertBefore) {
            this.setInsertPoint($insertParent, insertBefore);
            this.removeAndSwitch();
            this.insertAndSwitch();
        };

        this.formOpened = function(isdraft) {
            cForm.resetButtons();
            autosaveChecksEnabled = isdraft;
            AJS.$("#"+namePrefix + "AutosaveMsg").html("");
            if (autosaveChecksEnabled) {
                if (cForm.hasCommentId()) {
                    cForm.activateAutosaveButtons();
                }
                lastAutosaveDate = new Date();
                lastTextChangeDate = new Date();
                lastAutosaveText = cForm.getCommentText();
            }
        };

        this.formClosed = function() {
            autosaveChecksEnabled = false;
        };

        /**
         * Converts a date object to a string in the format of HH:MM [AM|PM]
         * @param d date object to stringify
         */
        var date2TimeOfDay = function (d) {
            var s = "AM";
            var h = d.getHours();
            var m = d.getMinutes();

            if (h == 12) {
                s = "PM";
            } else if (h > 12) {
                s = "PM";
                h = h - 12;
            }
            m = "" + m;
            if (m.length < 2) {
                m = "0" + m;
            }
            return h + ":" + m + " " + s;
        };

        this.autosaveTimerPing = function() {
            if (!autosaveChecksEnabled || commentator.commentLock || !that.isDisplaying()) {
                return;
            }

            var now = new Date();
            var sinceLastSave = now.getTime() - lastAutosaveDate.getTime();
            var sinceLastEdit = now.getTime() - lastTextChangeDate.getTime();

            if ((sinceLastSave < 30000) && (sinceLastEdit < 2000)) {
                // don't bother saving if they have edited in the last 2s
                // until 30s passes
                return;
            }

            var currentText = cForm.getCommentText();
            if (currentText == lastAutosaveText) {
                return;
            }

            if (/^\s*$/.test(currentText)) {
                return; // you can't save empty comments at the moment
            }

            lastAutosaveText = currentText;
            lastAutosaveDate = now;

            commentator.commentLock = true; //note: if you return after this line, make sure you release the lock

            cForm.toggleButtons(false);

            var done = function(resp) {
                cForm.toggleButtons(true);
                commentator.commentLock = false;

                if (resp.worked) {
                    AJS.$("#"+namePrefix + "AutosaveMsg").text("Autosaved at " + date2TimeOfDay(new Date()));
                    cForm.setFormVal("commentId", resp.comment.id);
                    commentator.createOrUpdateComment(resp);
                    commentator.updateCommentCount(resp);

                    cForm.activateAutosaveButtons();
                } else {
                    AJS.$("#"+namePrefix + "AutosaveMsg").text("problem autosaving");
                }
            };

            var autosaveFunc = eval(autosaveFuncName);
            autosaveFunc(permaId, cForm.getForm(), done); // NB: permaId is a global
            AJS.$("#"+namePrefix + "AutosaveMsg").text("Autosaving...");
        };

        this.autosaveTextChanged = function() {
            if (!autosaveChecksEnabled) {
                return;
            }
            var currentText = cForm.getCommentText();
            if (currentText == lastAutosaveText) {
                return;
            }

            lastTextChangeDate = new Date();

            var $autosaveMsg = AJS.$("#"+namePrefix + "AutosaveMsg");
            var msg = $autosaveMsg.text();
            if (msg != "" && !/\*$/.test(msg)) {
                // put an astrix at the end when it is unsaved
                $autosaveMsg.text(msg + " *");
            }
            msg = "changed, not saved";
        };
    };

    AJS.CRU.COMMENT.FORMS.CommentFormWrangler = CommentFormWrangler;
})();
