AJS.CRU.COMMENT.NAV = (function() {

    var currentCommentId = null;

    //current (comment) can be null
    var doScrollToComment = function (opts) {
        var sourceCommentId = opts.comment;  //comment is the div id like revisioncomment43
        var destinationComment = null;
        var inlineComment = /^inlinecomment/;
        var inlineReply = /^inlinereply/;
        if (commentator.showCommentsAbove()) {
            inlineComment = /^abovecomment/;
            inlineReply = /^abovereply/;
        }

        // cache opts values into local variables for faster lookups
        var nextThread = opts.nextThread;
        var destination = opts.destination;
        var skipReadComments = opts.skipReadComments;

        var commentDomIds = []; // List of comment ids to traverse over
        AJS.$.each(review.comments(), function() {
            if (this.frx() && this.frx().isFiltered()) {
                return true;
            }
            var commentid = this.domId();
            if (commentator.showCommentsAbove() && /^inline/.test(this.type())) { // if it's an inline comment and they're shown above, change the id
                commentid = commentid.replace('inline','above');
            }

            if (nextThread) {
                if (inlineComment.test(commentid) ||
                    /^revisioncomment[0-9]/.test(commentid) ||
                    /^generalcomment[0-9]/.test(commentid)
                    ) {
                    commentDomIds.push("#"+commentid);
                }
                // we need to make sure the current reply is also added to the comments
                // so we can find the next comment thread from the reply.
                else if(opts.comment && opts.comment == commentid) {
                    commentDomIds.push("#"+commentid);
                }
            }
            else if (inlineComment.test(commentid) ||
                inlineReply.test(commentid) ||
                /^revisioncomment[0-9]/.test(commentid) ||
                /^generalcomment[0-9]/.test(commentid) ||
                /^generalreply[0-9]/.test(commentid) ||
                /^revisionreply[0-9]/.test(commentid)
                ) {
                commentDomIds.push("#"+commentid);
            }
        } );
        // Convert the comments id list into a list of jquery objects
        var $commentDomElems = AJS.$(commentDomIds.join(","));

        if (destination == 'first' && !skipReadComments) {
            destinationComment = $commentDomElems[0];
        }
        else if(destination =='last' && !skipReadComments) {
            destinationComment = $commentDomElems[$commentDomElems.length-1];
        }
        else {
            // Find the source comment id in the list of comments
            var index = -1;
            var len = $commentDomElems.length;
            if (destination == 'last') {
                index = len;
            } else if (destination !== 'first') {
                for( var j= 0; j < len; j++ ) {
                    if( sourceCommentId == $commentDomElems[j].id ) {
                        index = j;
                        break;
                    }
                }
            }

            // If we're iterating over unread comments, we want to keep jumping comments
            // when the comment is marked as 'read'.
            do {
                if (destination == 'previous' || destination == 'last') {
                    if ($commentDomElems[index-1]) {
                        destinationComment = $commentDomElems[index-1];
                        index--;
                    } else {
                        destinationComment = skipReadComments ? currentCommentId : $commentDomElems[index];
                        break;
                    }
                } else if (destination == 'next' || destination == 'first') {
                    if ($commentDomElems[index+1]) {
                        destinationComment = $commentDomElems[index+1];
                        index++;
                    } else {
                        destinationComment = skipReadComments ? currentCommentId : $commentDomElems[index];
                        break;
                    }
                } else {
                    destinationComment = $commentDomElems[index];
                    break;
                }
                var commentId = res.commentIdFromAnchor(destinationComment.id);
                var commentStatus = review.comment(commentId).status();
            } while( skipReadComments && !(commentStatus == 'unread' || commentStatus == 'leaveUnread') );
        }
        if (!destinationComment || !destinationComment.id) {
            return false;
        }
        // ensure destination comment is visible
        var destId = destinationComment.id.replace(/[0-9]+/,'');
        if (!AJS.CRU.UI.isSingleFrxView() && destId != 'generalcomment' && destId != 'generalreply') {
            var $innerFrx = AJS.$(destinationComment).parents(".frxinner:first");
            // remove the leading 'frxinner' from the id
            AJS.CRU.FRX.expandFrx($innerFrx.attr('id').substring(8));
        }

        var frxScroller = AJS.CRU.FRX.scroller;
        // scroll to the comment's frx
        var scrollToCommentId = destinationComment.id.replace(destId,'');
        if (scrollToCommentId && review.comment(scrollToCommentId).frx()) {
            frxScroller.ignoreNextScroll();
            frxScroller.setCurrentElement(AJS.$('#frxouter' + review.comment(scrollToCommentId).frx().id())[0], true);
        } else {
            frxScroller.ignoreNextScroll();
            frxScroller.setCurrentElement(AJS.$('#generalComments')[0], true);
        }

        // Bypass the scroll tracker while we scroll, otherwise it will reclaim the
        // top-most (w.r.t. viewport) comment as the currentComment.
        commentScrollTracker.ignoreNextScroll();
        AJS.$('#frx-pane').scrollTo(destinationComment, {
            axis: 'y',
            offset: -100
        });
        res.setCurrentComment(destinationComment);
        commentScrollTracker.setCurrentElement(AJS.$(destinationComment).find('div.comment:first')[0]);

        return true;
    };

    var res = {
        getCurrentCommentId : function () {
            return currentCommentId;
        },

        /**
         * @param {String} opts.destination location to scroll to - may be <tt>first</tt>, <tt>last</tt>, <tt>previous</tt> or <tt>next</tt>.
         * @param {String} opts.comment the dom id of the comment to scroll to. This value is ignored if the <tt>opts.destination</tt> is <tt>first</tt> or <tt>last</tt>.
         * @param {Boolean} opts.skipReadComments if true, comments which have been marked as read will be passed over and ignored.
         * @param {Boolean} opts.nextThread if true, comment replies will be passed over and ignored.
         */
        scrollToComment : function (opts) {
            //if the next frx with comments isn't loaded load it first that way we know we'll get the next comment
            //small drawback is that if the next comment is in this frx there may be a delay
            var sourceCommentId = opts.comment ? res.commentIdFromAnchor(opts.comment) : "";
            var sourceFrxId = 0;
            if(sourceCommentId && review.comment(sourceCommentId)) {
                var frx = review.comment(sourceCommentId).frx();
                if(frx) {
                    sourceFrxId = frx.id();
                }
            }
            var nextFrxId = sourceFrxId;
            var frxIdsWithComments = review.visibleFrxIdsWithComments(); // skip filtered frxs
            var len = frxIdsWithComments.length;
            if(sourceFrxId != 0) {
                AJS.$.each( frxIdsWithComments, function(index) {
                    if( this == sourceFrxId ) {
                        if( index + 1 < len) {
                            nextFrxId = frxIdsWithComments[index+1];
                            return false; // break
                        }
                    }
                } );
            } else { //current comment isn't in an frx
                if(len > 0) {
                    nextFrxId = frxIdsWithComments[0];
                }
            }
            var nextFrx = review.frx(nextFrxId);
            if(nextFrx){ // if there is an frx with comments to load
                if(!nextFrx.isLoaded()) {
                    var onComp = function() {
                        doScrollToComment(opts);
                    };
                    AJS.CRU.FRX.AJAX.frxLoadDiff(nextFrxId, onComp);
                    return;
                }
            }
            doScrollToComment(opts);
        },

        setCurrentComment : function (destinationComment) {
            var shouldFlash = currentCommentId != destinationComment.id;
            if (currentCommentId != null) {
                commentator.markCommentRead(currentCommentId, false); // After moving away from the current comment, mark it as read
                AJS.$('#'+currentCommentId + " > .comment").removeClass("current_comment");
            }
            currentCommentId = destinationComment.id;
            var $comment = AJS.$('#'+currentCommentId + " > .comment");
            $comment.addClass("current_comment");
            if (shouldFlash && $comment.is(":visible")) { // effect on a hidden element shows it
                $comment.effect("highlight", {}, 800);
            }
        },

        checkCommentAnchor : function (loadOpts) {
            if (/^#c/.test(window.location.hash)) {
                var commentId = res.commentIdFromAnchor( window.location.hash.replace(/^#/,'') );
                var comment = review.comment(commentId);
                if (comment) {
                    var destId = comment.domId();
                    return res.gotoComment( { comment:destId }, loadOpts );
                }
            }
            return false;
        },

        nextComment : function ( current ) {
            current = current || res.getCurrentCommentId();
            var opts = { comment: current, destination: 'next' };
            res.gotoComment( opts, null );
        },

        prevComment : function ( current ) {
            current = current || res.getCurrentCommentId();
            var opts = { comment: current, destination: 'previous' };
            res.gotoComment( opts, null );
        },

        nextCommentThread : function ( current ) {
            current = current || res.getCurrentCommentId();
            var opts = { comment: current, destination: 'next', nextThread: true };
            res.gotoComment( opts, null );
        },

        previousCommentThread : function (current) {
            current = current || res.getCurrentCommentId();
            var opts = { comment: current, destination: 'previous', nextThread: true };
            res.gotoComment( opts, null );
        },

        /**
         * @param {String} opts.destination location to scroll to - may be <tt>first</tt>, <tt>last</tt>, <tt>previous</tt> or <tt>next</tt>.
         * @param {String} opts.comment the dom id of the comment to scroll to. This value is ignored if the <tt>opts.destination</tt> is <tt>first</tt> or <tt>last</tt>.
         * @param {Boolean} opts.skipReadComments if true, comments which have been marked as read will be passed over and ignored.
         * @param {Boolean} opts.nextThread if true, comment replies will be passed over and ignored.
         * @param {Object} loadOpts options for loading the comment frx if required.
         * @return the frxId of the frx of the comment if it has one
         */
        gotoComment : function (opts, loadOpts) {
            loadOpts = loadOpts || {};

            if( !opts.comment ) {
                opts.destination = 'first';
                res.scrollToComment(opts);
                return null;
            }
            var comment = review.comment( res.commentIdFromAnchor(opts.comment) );
            var frx = comment.frx();
            // if the comment doesn't live in an frx, just go to it, it must be loaded
            if( !frx || frx.isLoaded() ) {
                res.scrollToComment(opts);
                return frx ? frx.id() : null;
            }

            // otherwise, the comment lives in an frx and we must make sure it is loaded
            var frxId = frx.id();
            var onComp = function() {
                commentator.setCommentWidths(null,true);
                res.scrollToComment(opts);
            };
            
            if (comment.isHidden()) {
                // This diff makes it clear the comment is on deleted lines, and is similar
                // to the default diff for an FRX (i.e., showing the latest revision).
                var revisions = frx.frxRevisions();
                AJS.$.extend(loadOpts, {
                    fromRev: comment.toRevId(),
                    toRev: revisions[revisions.length - 1]
                });
            }

            AJS.CRU.FRX.AJAX.frxLoadDiff(frxId, onComp, loadOpts);
            return frxId;
        },

        // TODO Deprecate, everything should be using the Comment Object
        commentIdFromAnchor : function ( commentId ) {
            return commentId.replace( /^\D*/, "" );
        }
    };

    return res;
})();
