//TODO Arguments should be review-model objects, not ids.
var tetrisCommentController = (function() {

    var createAndShowHoverComment = function (cellId) {
        var $cell = AJS.$('#'+cellId.replace(',', '\\,'));
        var $hovercommentWrapper = AJS.$('#hovercommentWrapper'+cellId.replace(',', '\\,'));
        if ($hovercommentWrapper.length === 0) {
            $hovercommentWrapper = AJS.$('<div></div>')
                    .addClass('hovercommentWrapper')
                    .hover(
                        function() {openDropDown('hovercommentWrapper'+cellId);},
                        function() {closeDropDown();}
                    )
                    .hide()
                    .attr('id', 'hovercommentWrapper' + cellId);
        }

        var cellCommentArray = commentator.tetrisCellVisibleCommentCount[cellId];
        if (!cellCommentArray) {
            cellCommentArray = commentator.tetrisCellInvisibleCommentCount[cellId];
        }

        AJS.$.each(cellCommentArray, function() {
            //copy the above comment and make it a hover comment
            var $comment = AJS.$('#hovercomment' + this);
            if ($comment.length === 0) {
                $comment = AJS.$('#abovecomment'+this)
                        .clone()
                        .attr('id', 'hovercomment' + this);
                //remove comment buttons (edit, delete, etc)
                $comment.find('td.commentButton')
                        .replaceWith('<td>&nbsp;</td>');
                $comment.find('td.commentIssueKeyUnlinked')
                        .replaceWith('<td>&nbsp;</td>');
                //remove replyFormDivs
                $comment.find('#abovereplyFormDiv'+this)
                        .remove();
            }
            $hovercommentWrapper.append($comment);
            $comment.find('.comment').addClass('hover-comment');
        });

        $cell.append($hovercommentWrapper);
        openDropDown('hovercommentWrapper'+cellId.replace(',', '\\,'));
    };

    var res = {
        /**
         * Renders the comment markers for all comments in the specified FRX.
         */
        renderTetrisCommentMarkersForFrx : function (frxId) {
            AJS.$.each( review.inlineComments(), function() {
                if (this.frx().id() == frxId) {
                    res.renderTetrisCommentMarkersForComment(this.id());
                }
            } );
        },

        /**
         * Renders the comment markers for the given comment.
         * TODO: this method runs in O(n) where n == frx line count. We _MUST_ improve this.
         */
        renderTetrisCommentMarkersForComment : function (commentId) {
            var comment = review.comment(commentId);
            if (!comment) {
                return;
            }
            var frxId = comment.frx().id();

            var fromLines = [];
            var toLines = [];
            var className = "tetrisCommentVisible";
            var cellCounter = commentator.tetrisCellVisibleCommentCount;
            if (comment.isHidden()) {
                toLines = commentator.getCommentedLines(frxId, "", comment.gutterLine());
                if (toLines.length === 0) {
                    toLines.push(AJS.$('#'+frxId+'_last_line').attr('id', frxId+'_last_line')[0]);
                }
                className = "tetrisCommentHidden";
                cellCounter = commentator.tetrisCellInvisibleCommentCount;
            } else {
                if (comment.fromRevId()) {
                    fromLines = commentator.getCommentedLines(frxId, comment.fromLineRange(), "");
                }
                if (comment.toRevId()) {
                    toLines = commentator.getCommentedLines(frxId, "", comment.toLineRange());
                }
            }

            function updateCellCounters( lines, revId ) {
                AJS.$(lines).each( function() {
                    var cellId = this.id+"_"+revId;
                    if (!cellCounter[cellId]) {
                        cellCounter[cellId] = [];
                    }
                    if (AJS.$.inArray(commentId, cellCounter[cellId]) == -1) {
                        cellCounter[cellId].push(commentId);
                    }
                    var $cell = AJS.$('#'+cellId.replace(',', '\\,'));
                    $cell.addClass(className)
                        .unbind()
                        .hover(
                            function() {createAndShowHoverComment(cellId);},
                            function() {closeDropDown();}
                        );
                    if (className == 'tetrisCommentHidden') {
                        if ($cell.find('span').length != 1) {
                            AJS.$('<span>')
                                .html('&nbsp;')
                                .addClass(className)
                                .appendTo($cell);
                        }
                    } else {
                        $cell.addClass(comment.commentColor());
                    }
                } );
            }

            updateCellCounters(fromLines, comment.fromRevId());
            updateCellCounters(toLines, comment.toRevId());
        },

        deleteTetrisCommentMarkers : function (commentId) {
            var comment = review.comment(commentId);
            if (!comment) {
                return;
            }
            var frxId = comment.frx().id();
            var fromLines = [];
            var toLines = [];
            var className = "tetrisCommentVisible";
            var cellCounter = commentator.tetrisCellVisibleCommentCount;

            if (comment.isHidden()) {
                toLines = commentator.getCommentedLines(frxId, "", comment.gutterLine());
                if (toLines.length === 0) {
                    toLines.push(AJS.$('#'+frxId+'_last_line').attr('id', frxId+'_last_line')[0]);
                }
                className = "tetrisCommentHidden";
                cellCounter = commentator.tetrisCellInvisibleCommentCount;
            } else {
                if (comment.fromRevId()) {
                    fromLines = commentator.getCommentedLines(frxId, comment.fromLineRange(), "");
                }
                if (comment.toRevId()) {
                    toLines = commentator.getCommentedLines(frxId, "", comment.toLineRange());
                }
            }

            function updateCellCounters( lines, revId ) {
                AJS.$.each(lines, function() {
                    var cellId = this.id+"_"+revId;
                    if (!cellCounter[cellId]) {
                        return;
                    }

                    var index = AJS.$.inArray(commentId, cellCounter[cellId]);
                    if (index >= 0) {
                        cellCounter[cellId].splice(index, 1);
                    }
                    
                    if (cellCounter[cellId].length === 0) {
                        AJS.$('#'+cellId.replace(',', '\\,'))
                                .removeClass(className)
                                .removeClass(comment.commentColor())
                                .unbind();
                    }
                    AJS.$('#hovercommentWrapper'+cellId.replace(',', '\\,')).remove();
                    if (className == 'tetrisCommentHidden') {
                        AJS.$('#'+cellId.replace(',', '\\,')).find('span').remove();
                    }
                } );
            }

            updateCellCounters(fromLines, comment.fromRevId());
            updateCellCounters(toLines, comment.toRevId());
        }
    };
    return res;
})();
