AJS.CRU.COMMENT = (function() {
    var g_mousedown = false;
    var g_modeSelecting = false;
    var g_currentTable = null;
    var g_mouseDownFirstRow = null;
    var lastSelectedLine = null;
    var firstTimeDisplayingInlineForm = true;
    var frxPaneWidth = null;
    var commentDivPrefixes = ['general','revision','inline','above','defect'];

    var commentFormNS = AJS.CRU.COMMENT.FORMS;
    var scFormWrangler = new commentFormNS.CommentFormWrangler(new commentFormNS.GeneralCommentForm('simpleCommentForm','commentFormTemplate'), 'general', 'commentAjaxController.autoSaveGeneralComment');
    var rcFormWrangler = new commentFormNS.CommentFormWrangler(new commentFormNS.RevisionCommentForm('revisionCommentForm','revisionCommentFormTemplate'), 'revision', 'commentAjaxController.autoSaveRevisionComment');
    var fcFormWrangler = new commentFormNS.CommentFormWrangler(new commentFormNS.FileCommentForm('fileCommentForm','fileCommentFormTemplate'), 'file', 'commentAjaxController.autoSaveRevisionComment');
    var replyFormWrangler = new commentFormNS.CommentFormWrangler(new commentFormNS.ReplyCommentForm('replyCommentForm','replyFormTemplate'), 'reply', 'commentAjaxController.autoSaveReply');

    var tetrisCellVisibleCommentCount = {};
    var tetrisCellInvisibleCommentCount = {};

    var highlightedLines = [];
    var editingRCHandleId;

    var showInline = true;
    var showTop = false;
    var showRevisionComments = true;

    var selectTR = function (tr, select) {
        var $tr = AJS.$(tr);
        if ($tr.hasClass('commentableLine')) {
            $tr.toggleClass('lineHighlighted', select)
               .toggleClass('sourceLine', !select);
            if (!select) {
                $tr.removeClass('hoveringComment');
            }
        }
    };

    var clearSelectedRows = function () {
        g_modeSelecting = false;
        if(g_currentTable) {
            AJS.$(g_currentTable).find("tr.lineHighlighted").each(function() {
                 selectTR(this, false);
            });
        } else {
            AJS.$("tr.lineHighlighted").each(function() {
                 selectTR(this, false);
            });
        }
        rcFormWrangler.getCommentForm().clearComment();
        g_modeSelecting = false;
        g_currentTable = null;
        g_mouseDownFirstRow = null;
        firstTimeDisplayingInlineForm = true;
    };

    var convertLinesToRange = function (lines) {
        var lineStr = "";
        var count = 0;
        for (var i=0, len = lines.length; i < len; i++) {
            var line = lines[i];
            if (count != 0 && line == (parseInt(lines[i-1],10)+1)) {
                if (line != (parseInt(lines[i+1], 10) - 1)) {
                    lineStr += "-" + line;
                    count = 0;
                } else {
                    count++;
                }
            } else {
                if (i>0) { lineStr += ", "; }
                lineStr += line;
                count++;
            }
        }
        return lineStr;
    };

    var convertRangeToLines = function (ranges) {
        var lines = [];
        if( !ranges ) {
            return lines;
        }
        var range = ranges.split(",");
        for (var i=0, len = range.length; i < len; i++){
            var s = range[i].split("-");
            var s0 = s[0];
            if (s.length === 1) {
                lines.push(parseInt(s0,10));
            } else {
                for (var b=0; b <= (s[1]-s0); b++) {
                    lines.push(parseInt(s0, 10)+b);
                }
            }
        }
        return lines;
    };

    var insertDivComment = function (html, containerId){
        var $container = AJS.$("#"+containerId);
        if ($container.length === 1) {
            $container.append(html);
        }
    };

    var displayCommentForm = function (handleId, parentId, commentId, wrangler, exchanger) {
        var parent = AJS.$("#"+parentId);
        //display before setting the new values because safari/konqueror don't let you change the values
        //if not attached to the document unlike IE and FF
        if (exchanger) {
            exchanger();
        } else {
            wrangler.exchange(parent, null, handleId);
        }
        var isDraft = true;
        if(commentId) {
            var comment = review.comment(commentId);
            wrangler.getCommentForm().populateFromComment(comment);
            isDraft = comment.draft();
        } else {
            wrangler.getCommentForm().clearComment();
        }
        wrangler.formOpened(isDraft);
        wrangler.getCommentForm().setDefaultFocus();
    };

    var clearForm = function (wrangler) {
        wrangler.removeAndSwitch();
        wrangler.getCommentForm().clearComment();
    };

    /******************************************************************
     ******************** INLINE COMMENTS *****************************
     ******************************************************************/

    var checkInlineCommentBox = function (tableElement, forceFormExchange) {
        if(res.commentLock) {
            return false;
        }
        var toLinesSelected = [];
        var fromLinesSelected = [];
        var lastSelected = null;

        AJS.$(tableElement).children("tr.lineHighlighted:not(.hoveringComment)").each( function() {
            var num = this.id.split("_line")[1];
            if (num.charAt(0) === 'B') { // Both
                var lines = num.split(",");
                fromLinesSelected.push(lines[0].replace(/^Both/, ""));
                toLinesSelected.push(lines[1]);
            } else if (num.charAt(0) === 'F') { // From
                fromLinesSelected.push(num.replace(/^From/, ""));
            } else { // To
                toLinesSelected.push(num);
            }
            lastSelected = this;
        } );

        var commentForm = rcFormWrangler.getCommentForm();
        if (lastSelected != null) {
            var frxId = lastSelected.id.split('_')[0];
            //must display first before setting values or safari/konqueror bork
            var formHidden = commentForm.getForm().is(":hidden");
            if (forceFormExchange || formHidden) {
                if (formHidden) { // show it
                    rcFormWrangler.exchange(AJS.$(lastSelected).closest("table"), lastSelected, null);
                } else { // move it
                    rcFormWrangler.move(lastSelected);
                }
            }
            //only if displaying for the first time do we want to clear the comment text
            //so that if they click a line after entering text it doesn't clear
            if (firstTimeDisplayingInlineForm) {
                commentForm.clearComment();
                commentForm.setDefaultFocus();
                rcFormWrangler.formOpened(true);
                firstTimeDisplayingInlineForm = false;
            }
            commentForm.setFromLines(convertLinesToRange(fromLinesSelected));
            commentForm.setToLines(convertLinesToRange(toLinesSelected));
            commentForm.setFrxId(frxId);
            var frx = review.frx(frxId);
            commentForm.setFromFrxRevision(frx.visibleFromRevision());
            commentForm.setToFrxRevision(frx.visibleToRevision());
            if(rcFormWrangler.isDisplayingRow()){
                rcFormWrangler.setWidth(getInlineCommentFormWidth());
            } else {
                rcFormWrangler.unsetWidth();
            }
            lastSelectedLine = lastSelected;
        } else {
            //clean up
            if(commentForm.getTextBox().val() !== '') {
                var popupDeleteResponse = window.confirm("You've deselected all lines\n\n" +
                                                         "Delete the comment?\n");
                if (popupDeleteResponse) {
                    if (commentForm.hasCommentId()) {
                        commentator.deleteComment(commentForm.getCommentId(), permaId);
                    }
                    res.clearRevisionCommentBox();
                }
            } else {
                res.clearRevisionCommentBox();
            }
        }
        return false;
    };

    var insertAjaxRevisionComment = function (resp) {
        fcFormWrangler.removeAndSwitch();
        if (resp.worked) {

            var commentId = resp.comment.id;
            var oldComment = review.comment(commentId);

            // if we are editing, we want to replace the existing replies
            if (oldComment && oldComment.domIdExists()) {
                // commentHtml contains the edit div too, so just remove it
                AJS.$("#revisioncommentEdit" + commentId).remove();
                AJS.$("#revisioncomment" + commentId).replaceWith(resp.commentHtml);
            } else {
                AJS.$("#revision_comments_frxinner"+resp.frxId).append(resp.commentHtml);
            }
        }
    };

    var insertAjaxInlineComment = function(resp) {
        if (resp.worked) {
            res.clearRevisionCommentBox();
            var commentId = resp.comment.id;
            var $oldComment = AJS.$('#inlinecomment' + commentId);
            var isEdit = $oldComment.length === 1;

            var createNewRow = true;

            // comment exists, let's update it
            if (isEdit) {
                if (lastSelectedLine) {
                    var $row = $oldComment.closest("tr");
                    // check to see if the updated comment is on a different last line.
                    var $oldLastLine = $row.prevAll(".sourceLine:first");
                    if ($oldLastLine.length === 1) {
                        var sameLastLine = $oldLastLine.attr('id') === lastSelectedLine.id;
                        if (sameLastLine) {
                            $oldComment.parent().html(resp.inlineCommentHtml);
                            createNewRow = false;
                        } else {
                            $row.remove();
                        }
                    }
                    editingRCHandleId = null;
                } else {
                    // the parent is the td which holds the contents of the inline comment
                    $oldComment.parent().html(resp.inlineCommentHtml);
                    AJS.log("TODO: Check how this gets called")
                }
            }

            if (createNewRow) {
                if (lastSelectedLine) {
                    var frx = review.frx(resp.frxId);
                    var colSpan = frx.colspan();
                    var $cell = AJS.$("<td colspan='"+colSpan+"'></td>")
                                    .html(resp.inlineCommentHtml);
                    AJS.$("<tr class='comment-row'></tr>")
                        .append($cell)
                        .insertAfter(lastSelectedLine);
                }
            }
            if (isEdit) {
                // aboveCommentHtml contains the edit div too, so just remove it
                AJS.$("#abovecommentEdit" + commentId).remove();
                AJS.$("#abovecomment" + commentId).replaceWith(resp.aboveCommentHtml);
            } else {
                AJS.$("#inline_comments_frxinner" + resp.frxId).append(resp.aboveCommentHtml);
            }

            res.setCommentWidths('inlinecomment' + commentId, true);
        }
    };

    var leaveCommentUnread = function (commentId) {
        if (canChangeCommentStatus(commentId, 'leaveUnread')) {
            res.updateCommentReadStatus(commentId, permaId, false);
        }
    };

    var canChangeCommentStatus = function (commentId, status) {
        var comment = review.comment(commentId);
        var $comment = AJS.$(res.commentContainerDivs(commentId)).children('.comment');
        return review.writable() && !($comment.hasClass('readStatusLocked') || comment.status() === status);
    };
    
    var autosaveTimer = function () {
        rcFormWrangler.autosaveTimerPing();
        scFormWrangler.autosaveTimerPing();
        fcFormWrangler.autosaveTimerPing();
        replyFormWrangler.autosaveTimerPing();
        setTimeout(autosaveTimer, 10000); // every 10s
    };

    var calculateFrxPaneWidth = function () {
        // Leave room for the scrollbar.
        return AJS.$('#frx-pane').width() - 8;
    };
    
    var getInlineCommentFormWidth = function () {
        var commentFormIndent = 30;
        if (review.frxs()) {
            var currentWidth = calculateFrxPaneWidth();
            if (frxPaneWidth != currentWidth) {
                frxPaneWidth = currentWidth;
            }
        }
        return frxPaneWidth - commentFormIndent - 21;
    };

    var deleteComments = function (midFix,commentId) {
        AJS.$.each( commentDivPrefixes, function() {
            var $oldComment = AJS.$("#"+this+midFix+commentId);
            $oldComment.remove();
        });
        res.clearCommentedLines();
    };

    var updateRevisionSliderCounts = function (comment, totalDelta, unreadDelta) {
        if (comment.isInline()) {
            if (comment.fromRevId()) {
                updateCommentCountImpl('frxRev', comment.fromRevId(), totalDelta, true);
                updateUnreadCommentCountImpl('frxRev', comment.fromRevId(), unreadDelta, true);
            }
            if (comment.toRevId() && comment.fromRevId() !== comment.toRevId()) {
                updateCommentCountImpl('frxRev', comment.toRevId(), totalDelta, true);
                updateUnreadCommentCountImpl('frxRev', comment.toRevId(), unreadDelta, true);
            }
        }
    };

    // Comment counts
    var updateUnreadCommentCountImpl = function (idPrefix, idSuffix, unreadCommentCount, isDelta) {
        var $unread = AJS.$('#'+idPrefix+'UnreadCommentCount' + idSuffix);
        var unreadCount = isDelta ? (parseInt($unread.text(), 10) + unreadCommentCount) : unreadCommentCount;
        $unread.text(unreadCount);

        var $unreadWrapper = AJS.$('#'+idPrefix+'UnreadCommentCountWrapper'+idSuffix);
        if (unreadCount > 0) {
            $unreadWrapper.show();
        } else {
            $unreadWrapper.hide();
        }
    };

    var updateFRXCommentCountImpl = function (frxId, commentCount, isDelta) {
        updateCommentCountImpl('frx', frxId, commentCount, isDelta);
    };

    var updateCommentCountImpl = function (idPrefix, idSuffix, commentCount, isDelta) {
        var $total = AJS.$('#'+idPrefix+'CommentCount' + idSuffix);
        var currentTotal = parseInt($total.text(), 10);
        var newCommentCount = isDelta ? currentTotal + commentCount : commentCount;
        $total.text(newCommentCount);

        var $unreadSpan = AJS.$('#'+idPrefix+'UnreadComment' + idSuffix);
        if (newCommentCount > 0) {
            $total.show();
            $unreadSpan.show();
        } else {
            $total.hide();
            $unreadSpan.hide();
        }
    };

    var updateGeneralCommentCount = function (resp) {
        if (resp.worked) {
            var commentCountDelta = resp.generalCommentCountDelta;
            var unreadCommentCountDelta = resp.generalUnreadCommentCountDelta;
            updateCommentCountImpl('general', '', commentCountDelta, true );
            updateUnreadCommentCountImpl('general', '', unreadCommentCountDelta, true );
        }
    };

    var modifyIndicatorDisplay = function (midFix, commentId, value) {
        AJS.$.each( commentator.commentDivPrefixes, function() {
            AJS.$("#"+this+midFix+commentId).css('display',value);
        });
    };

    var res = {

        tetrisCellVisibleCommentCount : tetrisCellVisibleCommentCount,
        tetrisCellInvisibleCommentCount : tetrisCellInvisibleCommentCount,
        showCommentsInline : function () { return showInline; },
        showCommentsAbove : function () { return showTop; },
        showSource : true,
        commentDivPrefixes : commentDivPrefixes,
        g_pageCompletelyLoaded : false,
        commentLock : false,

        /******************************************************************
         ******************** HELPER METHODS ******************************
         ******************************************************************/

        showCommentedLines : function (frxId, fromLineRange, toLineRange){
            highlightedLines = res.getCommentedLines(frxId, fromLineRange, toLineRange);
            AJS.$(highlightedLines).addClass('lineHighlighted hoveringComment');
        },

        clearCommentedLines : function () {
            AJS.$(highlightedLines).removeClass('lineHighlighted hoveringComment');
            highlightedLines = [];
        },

        /******************************************************************
         ******************** COMMON METHODS ******************************
         ******************************************************************/

        updateFRXUnreadCommentCount : function (frxId, unreadCommentCount, isDelta) {
            updateUnreadCommentCountImpl('frx', frxId, unreadCommentCount, isDelta);
        },

        changeCommentStatus : function (commentId, newStatus, oldStatus) {
            AJS.$(res.commentContainerDivs(commentId))
                .children('.comment')
                .removeClass(oldStatus)
                .addClass(newStatus);
            review.comment(commentId).setStatus(newStatus);
        },

        displaySimpleCommentForm : function (handleId, parentId, commentId) {
            displayCommentForm(handleId, parentId, commentId, scFormWrangler);
        },

        createOrUpdateComment : function (resp) {
            if (resp.worked) {
                var commentResp = resp.comment;

                var comment = review.comment(commentResp.id);
                if (!comment) {
                    var position = 'general';
                    if (resp.replyToId) {
                        position = review.comment(resp.replyToId).position();
                    } else if (resp.frxId) {
                        if (resp.whole) {
                            position = 'revision';
                        } else {
                            position = 'inline';
                        }
                    }
                    var type = resp.replyToId ? 'reply' : 'comment';
                    var commentType = position + type;

                    comment = new Comment(commentResp.id, commentType);

                    if (resp.replyToId) {
                        var parent = review.comment(resp.replyToId);
                        comment.setReplyTo( parent );
                        parent.addReply(comment);
                    }

                    if (resp.frxId) {
                        var frx = review.frx(resp.frxId);
                        comment.setFrx(frx);
                        frx.addComment(comment);
                    }
                    comment.setReview(review);
                    review.addComment(comment);
                }
                comment.setMessage(commentResp.message)
                       .setToLineRange(commentResp.toLineRange)
                       .setFromLineRange(commentResp.fromLineRange)
                       .setGutterLine(commentResp.gutterLine)
                       .setToRevId(commentResp.toRevId)
                       .setFromRevId(commentResp.fromRevId)
                       .setDefect(commentResp.defect)
                       .setDraft(commentResp.draft)
                       .setMetrics(commentResp.metrics)
                       .setCommentColor(commentResp.commentColor)
                       .setIssueKey(commentResp.issueKey);

                return comment;
            }
        },

        updateCommentCount : function (resp) {
            if (resp.worked) {
                res.updateTotalCommentCount(resp.totalCommentCountDelta, true);
                res.updateTotalUnreadCommentCount(resp.totalUnreadCommentCountDelta, true);
                if (resp.frxId) {
                    res.updateFRXCommentCount(resp);
                } else {
                    updateGeneralCommentCount(resp);
                }
            }
        },

        updateTotalCommentCount : function (count, isDelta) {
            var $total = AJS.$("#totalCommentCount");
            var total;
            if (isDelta) {
                total = parseInt($total.text(), 10);
                total += count;
            } else {
                total = count;
            }

            $total.text(total);
        },

        updateTotalUnreadCommentCount : function (count, isDelta) {
            var $unread = AJS.$("#totalUnreadCommentCount");
            var unread;
            if (isDelta) {
                unread = parseInt($unread.text(), 10);
                unread += count;
            } else {
                unread = count;
            }

            AJS.$('#totalUnreadCommentCountWrapper')[unread > 0 ? 'show' : 'hide']();
            $unread.text(unread);
        },

        discardFileCommentBox : function (commentId, permaId) {
            res.clearFileCommentBox();
            commentAjaxController.deleteComment(commentId, permaId);
        },

        discardRevisionCommentBox : function (commentId, permaId) {
            res.clearRevisionCommentBox();
            commentAjaxController.deleteComment(commentId, permaId);
        },

        clearFileCommentBox : function () {
            clearForm(fcFormWrangler);
            fcFormWrangler.formClosed();
        },

        clearRevisionCommentBox : function () {
            if (rcFormWrangler.isDisplaying()) {
                //a little inefficient, but effective.
                rcFormWrangler.removeAndSwitch();
                clearSelectedRows();
                if(editingRCHandleId){
                    AJS.$("#"+editingRCHandleId).show();
                    editingRCHandleId = null;
                }
                rcFormWrangler.formClosed();
            }
        },

        /** Returns all occurences of the container divs. Needed due to inline/above. */
        commentContainerDivs : function (commentId) {
            var comment = review.comment(commentId);
            var domId = comment.domId();
            var domIds = ['#' + domId];
            if (/^inline/.test(comment.type())) {
                domIds.push('#' + domId.replace('inline', 'above') );
                domIds.push('.comment' + commentId);
            }
            return AJS.$(domIds.join(',')).get();
        },

        replyToComment : function (commentId) {
            var comment = review.comment(commentId);
            if (!comment) {
                return;
            }
            if (comment.isReply()) {
                comment = comment.replyTo(); // get the thread parent
            }
            var $commentElem = AJS.$("#" + comment.domId());
            $commentElem.find("div.comment:first div.comment-actions .replyToComment:first").click();
        },

        /******************************************************************
         ******************** COMMENT EDITING *****************************
         ******************************************************************/

        publishComment: function (commentId, permaId) {
            var done = function (commentId, resp) {
                if (resp.worked) {
                    modifyIndicatorDisplay("draftIndicator", commentId, "none");
                    modifyIndicatorDisplay("draftPublish", commentId, "none");
                    modifyIndicatorDisplay("starComment", commentId, "inline");
                    review.comment(commentId).setDraft(false);
                    AJS.$('.comment' + commentId + ' .comment').removeClass('draft');
                    commentator.updateCommentCount(resp);
                }
            };
            commentAjaxController.publishComment(commentId, permaId, done);
            return false;
        },

        /******************************************************************
         ******************** COMMENT DELETING ****************************
         ******************************************************************/

        deleteComment : function (commentId, permaId) {
            var done = function(resp) {
                if (resp.worked) {
                    var comment = review.comment(commentId);
                    //turn off all displayed comments with comment ID
                    deleteComments("comment", commentId); // TODO optimise
                    deleteComments("reply", commentId);   // TODO optimise
                    res.updateCommentCount(resp);
                    if (comment.isInline()) {
                        tetrisCommentController.deleteTetrisCommentMarkers(commentId);
                        updateRevisionSliderCounts(comment, -1, 0);
                    }

                    review.removeComment(comment);

                    commentScrollTracker.rescan();
                }
            };
            commentAjaxController.deleteComment(commentId, permaId, done);
            return false;
        },

        /******************************************************************
         ******************** REPLY COMMENTS ******************************
         ******************************************************************/

        insertAjaxReply : function (resp) {
            replyFormWrangler.removeAndSwitch();
            if (resp.worked) {

                var replyId = resp.comment.id;
                var idPrefix = resp.type;
                var isInline = idPrefix === 'inline';

                var oldComment = review.comment(replyId);

                // if we are editing, we want to replace the existing replies
                if (oldComment && oldComment.domIdExists()) {
                    // html contains the edit div too, so just remove it
                    AJS.$("#" + idPrefix + "commentEdit" + replyId).remove();
                    AJS.$("#" + idPrefix + "reply" + replyId).replaceWith(resp.html[idPrefix]);

                    if (isInline) {
                        AJS.$("#abovecommentEdit" + replyId).remove();
                        AJS.$("#abovereply" + replyId).replaceWith(resp.html['above']);
                    }
                } else {
                    var parentId = resp.replyToId;
                    AJS.$("#" + idPrefix + "replys" + parentId ).append(resp.html[idPrefix]);

                    if (isInline) {
                        AJS.$("#abovereplys" + parentId ).append(resp.html['above']);
                    }
                }

                res.createOrUpdateComment(resp);
                commentScrollTracker.rescan();
            }
        },

        displayReplyCommentForm : function (handleId, parentId, replyToId, commentId) {
            if (replyFormWrangler.isDisplaying() && AJS.$("#"+parentId)[0] === replyFormWrangler.getParent()[0]) {
                return;//already displaying here so do nothing
            }
            displayCommentForm(handleId, parentId, commentId, replyFormWrangler);
            //replyToId must be set or it ain't a reply
            replyFormWrangler.getCommentForm().setReplyToId(replyToId);
        },

        discardReplyCommentForm : function (commentId, permaId) {
            res.clearReplyCommentForm();
            commentAjaxController.deleteComment(commentId, permaId);
        },

        clearReplyCommentForm : function () {
            clearForm(replyFormWrangler);
            replyFormWrangler.formClosed();
        },

        /******************************************************************
         ******************** GENERAL COMMENTS ****************************
         ******************************************************************/

        insertAjaxGeneralComment : function (resp) {
            scFormWrangler.removeAndSwitch();
            if (resp.worked) {

                var commentId = resp.comment.id;
                var oldComment = review.comment(commentId);

                // if we are editing, we want to replace the existing replies
                if (oldComment && oldComment.domIdExists()) {
                    // html contains the edit div too, so just remove it
                    AJS.$("#generalcommentEdit" + commentId).remove();
                    AJS.$("#generalcomment" + commentId).replaceWith(resp.commentHtml);
                } else {
                    AJS.$("#generalCommentsInner").append(resp.commentHtml);
                }

                res.createOrUpdateComment(resp);
                res.updateCommentCount(resp);
                commentScrollTracker.rescan();
            }
        },

        discardGeneralCommentForm : function (commentId, permaId){
            res.clearGeneralCommentForm();
            commentAjaxController.deleteComment(commentId, permaId);
        },

        clearGeneralCommentForm : function () {
            clearForm(scFormWrangler);
            scFormWrangler.formClosed();
        },

        clearFileCommentForm : function () {
            clearForm(fcFormWrangler);
            fcFormWrangler.formClosed();
        },

        /******************************************************************
         ******************** REVISION COMMENTS ***************************
         ******************************************************************/

        displayFileCommentForm : function (handleId, parentId, frxId, commentId) {
            displayCommentForm(handleId, parentId, commentId, fcFormWrangler);
            fcFormWrangler.getCommentForm().setFrxId(frxId);

            fcFormWrangler.unsetWidth();
        },

        /**
         * display a comment form as a child of parent with the details in the comment object
         *
         * @param handleId the switch you want to hide while displaying the form
         * @param parentId div/containing block id to display form in
         * @param commentId comment dom id
        */
        displayRevisionCommentForm : function (handleId, parentId, frxId, commentId) {
            clearSelectedRows();

            var comment = review.comment(commentId);
            if (comment) {
                editingRCHandleId = handleId;
                var lines = res.getCommentedLines(frxId, comment.fromLineRange(), comment.toLineRange());
                AJS.$.each(lines, function(i) {
                    var $line = AJS.$(this);
                    if (i === 0) {
                        g_currentTable = $line.parent()[0];
                    }
                    selectTR(this, true);
                    lastSelectedLine = this;
                });
            }

            var exchanger = function () {
                rcFormWrangler.exchange(AJS.$(lastSelectedLine).closest("table"), lastSelectedLine, handleId);
            };
            displayCommentForm(handleId, parentId, commentId, rcFormWrangler, exchanger);

            var commentForm = rcFormWrangler.getCommentForm();
            commentForm.setFrxId(frxId);

            if (/^inline/.test(parentId)){
                rcFormWrangler.setWidth(getInlineCommentFormWidth());
                var frx = review.frx(frxId);
                commentForm.setFromFrxRevision(frx.visibleFromRevision());
                commentForm.setToFrxRevision(frx.visibleToRevision());
                firstTimeDisplayingInlineForm = false;
            } else {
                rcFormWrangler.unsetWidth();
                firstTimeDisplayingInlineForm = true;
            }
        },

        insertRevisionComment : function (resp) {
            if (resp.worked) {
                if (resp.whole) {
                    insertAjaxRevisionComment(resp);
                } else {
                    insertAjaxInlineComment(resp);
                }
                res.createOrUpdateComment(resp);
                res.updateCommentCount(resp);
                commentScrollTracker.rescan();
            }
        },

        /******************************************************************
         ******************** COMMENT READ STATUS *************************
         ******************************************************************/

        toggleCommentRead : function (commentId, force) {
            var comment = review.comment(commentId);
            if (!comment) {
                return;
            }
            var oldStatus = comment.status();
            if (oldStatus === 'read' || oldStatus === 'unread') {
                leaveCommentUnread(commentId);
            } else {
                res.markCommentRead(commentId, force);
            }
        },

        markCommentRead : function (commentDomId, force) {
            var commentId = AJS.CRU.COMMENT.NAV.commentIdFromAnchor(commentDomId);
            var comment = review.comment(commentId);
            if (!comment || comment.draft()) {
                return;
            }
            var leaveUnread = comment.status() === 'leaveUnread';
            if ( (!leaveUnread || force) && canChangeCommentStatus(commentId, 'read') ) {
                res.updateCommentReadStatus(commentId, permaId, true);
            }
        },

        leaveCommentUnread : function (commentId) {
            if (canChangeCommentStatus(commentId, 'leaveUnread')) {
                res.updateCommentReadStatus(commentId, permaId, false);
            }
        },

        markAllCommentsRead : function () {
            var updatedComments = [];
            AJS.$.each(review.comments(), function() {
                if (canChangeCommentStatus(this.id(), 'read')) {
                    updatedComments.push(this);
                }
            });

            var done = function(resp) {
                if (resp.worked) {
                    res.updateCommentCount(resp);
                    AJS.$.each(review.frxIdsWithComments(), function() {
                        res.updateFRXUnreadCommentCount( this, 0, false );
                    } );
                    AJS.$.each(updatedComments, function() {
                        res.changeCommentStatus(this.id(), 'read', this.status() );
                        if (this.isInline()) {
                            res.updateFrxRevisionUnreadCommentCount(this, -1);
                        }
                    });
                }
            };

            commentAjaxController.markAllCommentsRead(updatedComments, permaId, done);
            return false;
        },

        // Comment read status
        updateCommentReadStatus : function(commentId, permaId, markAsRead) {
            // Prevent simultaneous requests.
            var $comments = AJS.$(res.commentContainerDivs(commentId)).children('.comment')
                .addClass('readStatusLocked');
            var comment = review.comment(commentId);

            var oldStatus = comment.status();
            var newStatus = markAsRead ? 'read' : 'leaveUnread';

            var done = function(resp) {
                if (resp.worked) {
                    res.changeCommentStatus(commentId, newStatus, oldStatus);
                    commentator.updateCommentCount(resp);
                    if (comment.isInline()) {
                        commentator.updateFrxRevisionUnreadCommentCount(comment, resp.frxUnreadCommentCountDelta);
                    }
                }
                $comments.removeClass('readStatusLocked');
            };
            commentAjaxController.updateCommentReadStatus(commentId, permaId, markAsRead, done);
            return false;
        },

        /******************************************************************
         ******************** MISC METHODS    *****************************
         ******************************************************************/

        toggleComments : function (frxIds, commentIds, view) {
            AJS.$("#set-inline-comments, #set-above-comments, #set-hidden-comments").removeClass("selected");

            showRevisionComments = view !== 'none';
            showInline = view === 'inline';
            showTop = view === 'top';

            AJS.$.each(frxIds, function () {
                if (showRevisionComments) {
                    AJS.$('#revision_comments_frxinner'+this).show();
                } else {
                    AJS.$('#revision_comments_frxinner'+this).hide();
                }
                if (showTop) {
                    AJS.$('#inline_comments_frxinner'+this).show();
                } else {
                    AJS.$('#inline_comments_frxinner'+this).hide();
                }
            });

            AJS.$.each(commentIds, function () {
                if (showInline) {
                    AJS.$('#inlinecomment' + this).show();
                } else {
                    AJS.$('#inlinecomment' + this).hide();
                }
            });

            if (showInline && !res.showSource) {
                AJS.CRU.FRX.toggleSource(frxIds, commentIds);
            }

            AJS.$("#set-inline-comments").toggleClass("selected", showInline);
            AJS.$("#set-above-comments").toggleClass("selected", showTop);
            AJS.$("#set-hidden-comments").toggleClass("selected", !showRevisionComments);
        },

        /**
         * Returns a list of row objects which are to commented
         * TODO: optimise by not iterating over _every_ line of the frx, and by checking ids
         */
        getCommentedLines : function (frxId, fromLineRange, toLineRange){
            var commentedLines = [];

            if (!g_modeSelecting && (fromLineRange || toLineRange ) ) {
                highlightedLines = [];

                var $rows = AJS.$('#sourceTable'+frxId).find("tr.sourceLine");

                var toLines = convertRangeToLines(toLineRange);
                var fromLines = convertRangeToLines(fromLineRange);
                var toLinesLen = toLines.length;
                var fromLinesLen = fromLines.length;
                var toLine = 0;
                var fromLine = 0;

                $rows.each( function() {
                    var a = 0;
                    var lineId = this.id;
                    if( lineId ) {
                        var num = lineId.split("_line")[1];
                        if (num.charAt(0) === 'B') {
                            var lines = num.split(",");
                            fromLine = lines[0].replace(/^Both/, "");
                            toLine = lines[1];
                            var pushed = false;
                            for (a=0; a < toLinesLen; a++){
                                if (toLine == toLines[a]) {
                                    commentedLines.push(this);
                                    pushed = true;
                                    break;
                                }
                            }
                            for (a=0; !pushed && a < fromLinesLen; a++){
                                if (fromLine == fromLines[a]) {
                                    commentedLines.push(this);
                                    break;
                                }
                            }
                        } else if (num.charAt(0) === 'F') {
                            fromLine = num.replace(/^From/, "");
                            for (a=0 ; a < fromLinesLen; a++){
                                if (fromLine == fromLines[a]) {
                                    commentedLines.push(this);
                                    break;
                                }
                            }
                        } else {
                            toLine = num;
                            for (a=0 ; a < toLinesLen; a++){
                                if (toLine == toLines[a]) {
                                    commentedLines.push(this);
                                    break;
                                }
                            }
                        }
                    }
                } );
            }
            return commentedLines;
        },


        /**
         * set comment widths to the browser window width instead of the width of the overflowing div
         * it is contained in
         * @param commentId (optional) the id of the individual comment you wish to set the width of as used in ajax updated comments
         * @param forceUpdate (optional) force the comment widths to be set even if the width hasn't changed as used in ajax updates of frx's
         */
        setCommentWidths : function (commentId, forceUpdate) {
            var $commentToUpdate;
            if( commentId ) {
                $commentToUpdate = AJS.$("#"+commentId);
            }
            var commentIndent = 19;
            var replyIndent = 49;
            if (review.frxs()) {
                var currentWidth = calculateFrxPaneWidth();
                if ($commentToUpdate) {
                    $commentToUpdate.width(frxPaneWidth - commentIndent);
                } else if (forceUpdate || frxPaneWidth != currentWidth || !res.g_pageCompletelyLoaded) {
                    frxPaneWidth = currentWidth;
                    AJS.$.each(review.inlineComments(), function (i, comment) {
                        AJS.$('#' + comment.domId()).width(frxPaneWidth - (comment.isReply() ? replyIndent : commentIndent));
                    });
                    if (rcFormWrangler.isDisplaying() && /^inline/.test(rcFormWrangler.getParent().id)) {
                        rcFormWrangler.setWidth(getInlineCommentFormWidth());
                    }
                }
            }
            return true;
        },

        updateFRXCommentCount : function (resp) {
            if (resp.worked) {
                var frxId = resp.frxId;
                var frxCommentCountDelta = resp.frxCommentCountDelta;
                var frxUnreadCommentCountDelta = resp.frxUnreadCommentCountDelta;
                updateFRXCommentCountImpl(frxId, frxCommentCountDelta, true );
                res.updateFRXUnreadCommentCount(frxId, frxUnreadCommentCountDelta, true );
                if (resp.comment) {
                    var comment = review.comment(resp.comment.id);
                    updateRevisionSliderCounts(comment, frxCommentCountDelta, frxUnreadCommentCountDelta);
                }
            }
        },

        updateFrxRevisionUnreadCommentCount: function (comment, unreadDelta) {
            updateRevisionSliderCounts(comment, 0, unreadDelta);
        },

        selectLine_down : function (event, row, dontActuallySelectThisRow) {
            if (event.altKey) {
                return true;
            }
            var $row = AJS.$(row);
            var $parent = $row.parents(":first");
            var parent = $parent[0];
            g_mousedown = true;
            if (g_currentTable !== parent) {
                if (g_currentTable) {
                    clearSelectedRows();
                }
                g_currentTable = parent;
            }
            g_modeSelecting = !$row.hasClass("lineHighlighted");
            g_mouseDownFirstRow = row;
            if (!dontActuallySelectThisRow) {
                selectTR(row, g_modeSelecting );
                return false;
            } else {
                return true;
            }
        },

        selectLine_over : function (event, obj){
            if (event.altKey) {
                return true;
            }
            if (g_mouseDownFirstRow != null) {
                selectTR(g_mouseDownFirstRow, g_modeSelecting);
            }
            if (g_mousedown) {
                selectTR(obj, g_modeSelecting);
                return false;
            }
        },

        selectLine_up : function (event, obj) {
            if(event.altKey) {
                return true;
            }
            var $tableElement = AJS.$(obj).parent();
            checkInlineCommentBox($tableElement, true);
            g_mouseDownFirstRow = null;
            return false;
        },

        commentFromInnerElement : function (button) {
            var id = AJS.$(button).closest(".comment-container")
                        .attr("id")
                        .replace(/^\D*/,"");
            return review.comment(id);
        },

        syncCommentButtons : function () {
            // Sync the comment nav buttons
            var hasUnread = review.unreadComments().length > 0;
            AJS.$("#mark-comments-read-button").toggleClass("disabled", !hasUnread);
            // If we have unread comments, then we obviously have comments...
            if (hasUnread) {
                AJS.$("#frxs div.toolbar .prev-comment-button").removeClass("disabled");
                AJS.$("#frxs div.toolbar .next-comment-button").removeClass("disabled");
            } else {
                // otherwise check if we actually have comments
                var hasComments = review.comments().length > 0;
                AJS.$("#frxs div.toolbar .prev-comment-button").toggleClass("disabled", !hasComments);
                AJS.$("#frxs div.toolbar .next-comment-button").toggleClass("disabled", !hasComments);
            }
        }
    };

    AJS.$(document).ready( function() {
        AJS.$("body.crucible").mouseup(function() {
            g_mousedown = false;
        });

        setTimeout(autosaveTimer, 5000); // 5s after page load

        res.syncCommentButtons();
    });

    return res;
})();

var commentator = AJS.CRU.COMMENT;
