AJS.CRU.FRX.AJAX = (function() {
    var setFrxRevisionParams = function (frxId, from, to) {
        to = to || "";
        from = from || "";
        review
            .frx(frxId)
                .setVisibleFromRevision(from)
                .setVisibleToRevision(to);
    };

    var frxLoadAsyncDiff = function (i, frxIds, loadOpts) {
        if ((i < frxIds.length) && (i < 100)) { // load and chain
            var frxId = frxIds[i];
            var onComp = function() {
                frxLoadAsyncDiff(i + 1, frxIds, loadOpts); // chain
                commentator.setCommentWidths(); // TODO fixme - resets widths of _all_ comments
            };
            res.frxLoadDiff(frxId, onComp, loadOpts); // load
        } else { // when finished chaining
            if (AJS.CRU.FRX.postPostLoadFunc) {
                AJS.CRU.FRX.postPostLoadFunc();
            }
            AJS.$("#show_source_button").removeClass("disabled").addClass("selected").children("a").click(function () {
                AJS.CRU.FRX.toggleSource(review.frxIds(), review.inlineCommentIds());
            });
        }
    };

    var rebindEventsAndRereshComments = function (frxId) {
        tetrisCommentController.renderTetrisCommentMarkersForFrx(frxId);
        commentScrollTracker.rescan();
        commentAjaxController.loadCommentIssueStatuses(frxId);
    };

    var res = {
        frxRevisionParams : function (frxId) {
            var frx = review.frx(frxId);
            var params = "";
            if (!frx) {
                return params;
            }
            if( frx.visibleFromRevision() ) {
                params += "&fromRev=" + frx.visibleFromRevision();
            }
            params += "&toRev=" + frx.visibleToRevision();
            return params;
        },

        updateFrxView : function (from, to, frxId) {
            setFrxRevisionParams(frxId, from, to);
            var oldInnerHTML = AJS.$('#frxouter' + frxId).html();
            AJS.CRU.FRX.dimFrx(frxId);
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) + '/updateFrxViewAjax';
            var pars = "frxid=" + frxId + res.frxRevisionParams(frxId) + AJS.CRU.FRX.diffParams(frxId);
            var elementIdToUpdate = 'frxouter' + frxId;
            var onComp = function(origRequest) {
                if (!origRequest.worked) {
                    // if the request didn't work, make sure the spinner
                    // is taken out, and the old text restored.
                    AJS.$('#frxouter' + frxId).html(oldInnerHTML);
                } else {
                    res.reloadFrxModel(frxId, origRequest, true);
                    commentator.setCommentWidths(null, true);
                    res.updateFrxDiffControls(frxId, origRequest);
                    rebindEventsAndRereshComments(frxId);
                }
            };
            AJS.FECRU.AJAX.ajaxUpdate(url, pars, elementIdToUpdate, onComp);
            return false;
        },

        reloadFrxModel: function (frxId, resp, keepSliderRevs) {
            var oldFrx = review.frx(frxId);
            var isExpanded = oldFrx.isExpanded();
            if (keepSliderRevs) {
                var sliderFrxRevisions = oldFrx.getSliderFrxRevisions();
            }

            eval('var frxModelLoader = ' + resp.frxModelLoader);
            frxModelLoader();

            var newFrx = review.frx(frxId);
            newFrx
                .setLoaded(true)
                .setExpanded(isExpanded);
            if (keepSliderRevs) {
                newFrx.setSliderFrxRevisions(sliderFrxRevisions);
            }
        },

        updateFrxDiffControls: function (frxId, frxAjaxResp) {
            AJS.$('#diffOptionsFormPlaceholder' + frxId).html(frxAjaxResp.diff);
            // reset the fisheye diff link
            var toRev = review.frx(frxId).visibleToSCMRevision();
            var fromRev = review.frx(frxId).visibleFromSCMRevision();
            if (toRev === fromRev) {
                AJS.$('#fisheyeDiffUrl' + frxId).hide();
            } else {
                AJS.$('#fisheyeDiffUrl' + frxId + ' a')
                        .attr('href', AJS.$('#baseFisheyeDiffUrl' + frxId).val() + '?r1=' + fromRev + '&r2=' + toRev);
                AJS.$('#fisheyeDiffUrl' + frxId).show();
            }
            // Rebind the diff dropdown menus.
            AJS.FECRU.UI.setDropdowns();
        },

        updateFrxSlider: function (frxId, frxAjaxResp) {
            var $sliderDiv = AJS.$('#frxSliderDiv-' + frxId);
            $sliderDiv.html(frxAjaxResp.slider);
            // Sliders in the edit content panel of manage files should be disabled
            if ($sliderDiv.hasClass('editContent')) {
                AJS.$('#frxSlider_'+frxId).slider('disable');
            }
        },

        frxLoadDiff : function (frxId, onCompleteFunc, loadOpts) {
            loadOpts = loadOpts || {};
            
            var frx = review.frx(frxId);
            var fromRev = loadOpts.fromRev || frx.visibleFromRevision();
            var toRev = loadOpts.toRev || frx.visibleToRevision();
            
            var changeDiff = !(fromRev === frx.visibleFromRevision() && toRev === frx.visibleToRevision());
            var shouldReload = !frx.isLoaded() || loadOpts.isForcedReload || changeDiff;

            var onComplete = function (resp) {
                if (resp && resp.worked && shouldReload) {
                    res.reloadFrxModel(frxId, resp, false);
                    res.updateFrxDiffControls(frxId, resp);
                    res.updateFrxSlider(frxId, resp);
                    rebindEventsAndRereshComments(frxId);
                }
                if (onCompleteFunc) {
                    onCompleteFunc();
                }
            };


            if (shouldReload) {
                AJS.CRU.FRX.dimFrx(frxId);
                var elementIdToUpdate = 'frxinner' + frxId;
                var url = AJS.CRU.UTIL.jsonUrlBase(permaId) +'/loadFrxAjax';
                var diffModeOpts = loadOpts.diffMode || "";
                var noCacheRand = Math.floor(Math.random() * 999999);
                setFrxRevisionParams(frxId, fromRev, toRev);
                var pars = 'frxid=' + frxId + '&dontCache=' + noCacheRand + diffModeOpts + res.frxRevisionParams(frxId);
                frx.setLoaded(true);
                AJS.FECRU.AJAX.ajaxUpdate(url, pars, elementIdToUpdate, onComplete);
            } else {
                onComplete(); //allow the onComplete to finish despite not loading.
            }
        },

        frxLazyLoad : function (frxArray, loadOpts) {
            frxLoadAsyncDiff(0, frxArray, loadOpts);
        },

        markFRXComplete : function (frxId, complete) {
            if (!review.writable()) {
                return;
            }
            var frx = review.frx(frxId);
            if (!frx || frx.isComplete() === complete) {
                return;
            }

            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) +'/completeFRXAjax/';
            var params = "frxId=" + frxId + "&complete=" + complete;

            AJS.$('#notDone' + frxId).hide();
            AJS.$('#done' + frxId).hide();
            AJS.$('#busy' + frxId).css("display","inline");

            var done = function(resp) {
                AJS.$('#busy' + frxId).hide();
                var imgId = "#notDone" + frxId;
                var complete = false;
                if (resp.value == 1) {
                    imgId = "#done" + frxId;
                    complete = true;
                }
                AJS.$(imgId).show();
                var toggleCompleteSelectors = "#frx-list-item" + frxId + " > span.frx," +
                                              "#frx-list-item" + frxId + " > span.folder," +
                                              "#frxouter" + frxId;
                var $frxElems = AJS.$(toggleCompleteSelectors);
                if (complete) {
                    $frxElems.removeClass("frx-incomplete")
                             .addClass("frx-complete");
                } else {
                    $frxElems.addClass("frx-incomplete")
                             .removeClass("frx-complete");
                }
                frx.setComplete(complete);
                if (complete) {
                    var d1 = 0;
                    var d2 = 0;
                    AJS.$.each( review.frx(frxId).comments(), function() {
                        if (this.status() === 'unread') {
                            AJS.CRU.COMMENT.changeCommentStatus(this.id(), 'read', this.status() );
                            if (this.isInline()) {
                                AJS.CRU.COMMENT.updateFrxRevisionUnreadCommentCount(this, -1);
                            }
                            d2--;
                        }
                    });
                    AJS.CRU.COMMENT.updateFRXUnreadCommentCount(frxId, d2, true);
                    AJS.CRU.COMMENT.updateTotalUnreadCommentCount(d2, true);
                }
            };
            AJS.FECRU.AJAX.ajaxDo(url, params, done);
            return false;
        },

        toggleSourceType : function (permaId, frxId, showAnnotation, params, handler) {
            AJS.CRU.FRX.dimFrx(frxId);
            var url = AJS.CRU.UTIL.jsonUrlBase(permaId) +'/toggleSourceTypeAjax';
            var showAnnotationStr = '';
            var $dof = AJS.$('#diffOptionsForm');
            var $diffContextToggle = AJS.$('#diffControlsToggle' + frxId);
            var elementIdToUpdate = 'frxouter' + frxId;
            var onComp = function(origRequest) {
                if (!origRequest.worked) {
                    // if the request didn't work, make sure the spinner
                    // is taken out, and the old text restored.
                    AJS.$('#frxouter' + frxId).html(oldInnerHTML);
                } else {
                    res.reloadFrxModel(frxId, origRequest, true);
                    commentator.setCommentWidths(null, true);
                    res.updateFrxDiffControls(frxId, origRequest);
                    rebindEventsAndRereshComments(frxId);
                    if (handler) {
                        handler(origRequest);
                    }
                }
            };
            if (showAnnotation) {
                showAnnotationStr = '&showAnnotation=true';
                $dof.hide();
                $diffContextToggle.hide();
            } else {
                $dof.show();
                $diffContextToggle.show();
            }
            var pars = 'frxid=' + frxId + showAnnotationStr + params;
            AJS.FECRU.AJAX.ajaxUpdate(url, pars, elementIdToUpdate, onComp);
            return false;
        }
    };
    return res;
})();
