AJS.CRU.FRX.NAV = {};

(function() {

    AJS.CRU.FRX.NAV.getCurrentFrxId = function () {
        var current = AJS.CRU.FRX.scroller.getCurrentElement();
        return current ? current.id.replace('frxouter', '') : '';
    };

    var defaultOpts = {
        destination : 'first',
        skipCompleteFrxs : false
    };

    /**
     * @param {String} opts.destination location to scroll to - may be <tt>first</tt>, <tt>last</tt>, <tt>previous</tt> or <tt>next</tt>.
     * @param {String} opts.frxId the frx id of the frx to scroll to. This value is ignored if the <tt>opts.destination</tt> is <tt>first</tt> or <tt>last</tt>.
     * @param {Boolean} opts.skipCompleteFrxs if true, frxs which have been marked as complete will be passed over and ignored.
     */
    AJS.CRU.FRX.NAV.gotoFrx = function (opts) {
        var frx = review.frx(opts.frxId);
        // if the frx is loaded, just go to it
        if( !frx || frx.isLoaded() ) {
            scrollToFrx(opts);
            return;
        }

        // otherwise, we must make sure to load the frx first
        var onComp = function() {
            commentator.setCommentWidths(null,true);
            scrollToFrx(opts);
        };
        AJS.CRU.FRX.AJAX.frxLoadDiff(opts.frxId, onComp);
    };

    AJS.CRU.FRX.NAV.checkFrxAnchor = function () {
        var frxId;
        if (/^#CFR-\d+/.test(window.location.hash)) {
            frxId = window.location.hash.replace(/^#CFR-(\d+).*$/, '$1');
            AJS.CRU.FRX.NAV.gotoFrx({ frxId: frxId , destination: '' });
            return frxId;
        }
    };

    var scrollToFrx = function (settings) {
        var opts = {};

        AJS.$.extend(opts, defaultOpts, settings);

        var frxIds = AJS.$.merge(["generalComments"],
                AJS.$.grep(review.frxIds(), function (frxId) {
                    return !review.frx(frxId).isFiltered();
                })
            );

        if (!frxIds || frxIds.length === 0) {
            return;
        }

        var destinationFrxId = null;
        var destination = opts.destination;
        var skipCompleteFrxs = opts.skipCompleteFrxs;

        if (destination === 'first' && !skipCompleteFrxs) {
            destinationFrxId = frxIds[0];
        } else if (destination === 'last' && !skipCompleteFrxs) {
            destinationFrxId = frxIds[frxIds.length-1];
        } else {
            // Find the source frx index in the list of frxs
            var index = -1;
            if (destination === 'last') {
                index = frxIds.length;
            } else if (destination !== 'first') {
                for( var j = 0, len = frxIds.length; j < len; j++ ) {
                    if( opts.frxId == frxIds[j] ) {
                        index = j;
                        break;
                    }
                }
            }

            // If we're iterating over unread comments, we want to keep jumping comments
            // when the comment is marked as 'read'.
            do {
                if (destination == 'previous' || destination == 'last') {
                    if (frxIds[index-1]) {
                        destinationFrxId = frxIds[index-1];
                        index--;
                    } else {
                        destinationFrxId = skipCompleteFrxs ? destinationFrxId : frxIds[index];
                        break;
                    }
                } else if (destination == 'next' || destination == 'first') {
                    if (frxIds[index+1]) {
                        destinationFrxId = frxIds[index+1];
                        index++;
                    } else {
                        destinationFrxId = skipCompleteFrxs ? destinationFrxId : frxIds[index];
                        break;
                    }
                } else {
                    destinationFrxId = frxIds[index];
                    break;
                }
                var frx = review.frx(destinationFrxId);
                var isFrxComplete = frx && frx.isComplete();
            } while( skipCompleteFrxs && isFrxComplete );
        }

        if (!destinationFrxId) {
            return;
        }

        AJS.CRU.FRX.scrollFrxPane(destinationFrxId);
    };
})();
