AJS.$(document).ready(function () {

    var CNAV = AJS.CRU.COMMENT.NAV;

    // Disables key handling when any dialog is showing.
    function keyHandler(f) {
        return function () {
            AJS.CRU.REVIEW.UTIL.resetInactivityTimer();
            if (!AJS.CRU.UTIL.isAnyDialogShowing()) {
                return f.apply(this, arguments);
            }
        };
    }

    function commentNavigator(successor, opts, init) {
        opts = opts || {};
        init = init || successor;

        return keyHandler(function () {
            var commentId = CNAV.getCurrentCommentId();
            CNAV.gotoComment(AJS.$.extend({
                comment: commentId,
                destination: commentId ? successor : init
            }, opts), null);
            // stop the event propagating
            return false;
        });
    }

    function nextComment(opts) {
        return commentNavigator('next', opts, 'first');
    }

    function previousComment(opts) {
        return commentNavigator('previous', opts, 'last');
    }

    function frxNavigator(successor, opts, init) {
        opts = opts || {};
        init = init || successor;

        return keyHandler(function () {
            var frxId = AJS.CRU.FRX.NAV.getCurrentFrxId();
            AJS.CRU.FRX.NAV.gotoFrx(AJS.$.extend({
                frxId: frxId,
                destination: frxId ? successor : init
            }, opts));
        });
    }

    function nextFrx(opts) {
        return frxNavigator('next', opts, 'first');
    }

    function previousFrx(opts) {
        return frxNavigator('previous', opts, 'last');
    }

    function currentCommentId() {
        return CNAV.commentIdFromAnchor(CNAV.getCurrentCommentId());
    }

    function setFileReviewedImpl(frx, reviewed) {
        AJS.CRU.FRX.AJAX.markFRXComplete(frx.id(), reviewed);
    }

    function toggleFileReviewed() {
        return keyHandler(function () {
            if (review.writable()) {
                var frxId = AJS.CRU.FRX.NAV.getCurrentFrxId();
                var frx = review.frx(frxId);
                if (frx) {
                    setFileReviewedImpl(frx, !frx.isComplete());
                }
            }
            return false;
        });
    }

    function setFileReviewedAndMoveOn() {
        return keyHandler(function () {
            if (review.writable()) {
                var frxId = AJS.CRU.FRX.NAV.getCurrentFrxId();
                var nextFrx = function () {
                    AJS.CRU.FRX.NAV.gotoFrx({
                        frxId: frxId,
                        destination: 'next',
                        skipCompleteFrxs: true
                    });
                };
                if (frxId === 'generalComments') {
                    nextFrx();
                } else {
                    var frx = review.frx(frxId);
                    if (frx) {
                        setFileReviewedImpl(frx, true);
                        AJS.CRU.FRX.scroller.ignoreNextScroll();
                        AJS.CRU.FRX.collapseFrxs(frxId);
                        nextFrx();
                    }
                }
            }
            return false;
        });
    }

    var categories = [
      { category: 'Comment Navigation',
        shortcuts: [
          { keys: ['p'],
            desc: 'previous comment',
            action: previousComment() },
          { keys: ['n'],
            desc: 'next comment',
            action: nextComment() },

          { keys: ['shift+p'],
            desc: 'first comment',
            action: commentNavigator('first') },
          { keys: ['shift+n'],
            desc: 'last comment',
            action: commentNavigator('last') },

          { keys: ['h'],
            desc: 'previous thread (skips replies)',
            action: previousComment({ nextThread: true }) },
          { keys: ['l'],
            desc: 'next thread (skips replies)',
            action: nextComment({ nextThread: true }) },

          { keys: ['['],
            desc: 'previous unread comment',
            action: previousComment({ skipReadComments: true }) },
          { keys: [']'],
            desc: 'next unread comment',
            action: nextComment({ skipReadComments: true }) },

          { keys: ['r'],
            desc: 'reply to comment',
            action: keyHandler(function () {
                commentator.replyToComment(currentCommentId());
                return false;
            }) },

          { keys: ['m'],
            desc: 'toggle comment read/unread',
            action: keyHandler(function () {
                commentator.toggleCommentRead(currentCommentId(), true);
                return false;
            }) }
        ] },

      { category: 'File Navigation',
        shortcuts: [
            // note: if you change these values, also change the tooltips where relevant
          { keys: ['k'],
            desc: 'previous file',
            action: previousFrx() },
          { keys: ['j'],
            desc: 'next file',
            action: nextFrx() },

          { keys: ['shift+k'],
            desc: 'first file',
            action: frxNavigator('first') },
          { keys: ['shift+j'],
            desc: 'last file',
            action: frxNavigator('last') },

          { keys: ['i'],
            desc: 'previous not reviewed file',
            action: previousFrx({ skipCompleteFrxs: true }) },
          { keys: ['u'],
            desc: 'next not reviewed file',
            action: nextFrx({ skipCompleteFrxs: true }) },

          { keys: ['y'],
            desc: 'set file reviewed and go to next unreviewed file',
            action: setFileReviewedAndMoveOn() },
          { keys: ['shift+y'],
            desc: 'toggle file reviewed/not reviewed',
            action: toggleFileReviewed() },

          { keys: ['e'],
            desc: 'expand current file',
            action: keyHandler(function () {
                AJS.CRU.FRX.expandFrxs(AJS.CRU.FRX.NAV.getCurrentFrxId());
            }) },
          { keys: ['c'],
            desc: 'collapse current file',
            action: keyHandler(function () {
                AJS.CRU.FRX.collapseFrxs(AJS.CRU.FRX.NAV.getCurrentFrxId());
            }) },

          { keys: ['shift+e'],
            desc: 'expand all files',
            action: keyHandler(function () {
                AJS.CRU.FRX.expandFrxs(review.frxIds());
            }) },
          { keys: ['shift+c'],
            desc: 'collapse all files',
            action: keyHandler(function () {
                AJS.CRU.FRX.collapseFrxs(review.frxIds());
            }) }
        ] },

      { category: 'Reviews',
        shortcuts: [
          { keys: ['alt'],
            desc: 'hold down then click and drag to select source line contents',
            action: keyHandler(function () {
                // handled by commentator.js (selectLine_* functions)
            }) },
          { keys: ['shift+f'],
            desc: 'toggle full screen mode',
            action: keyHandler(function () {
                AJS.CRU.UI.toggleFullscreen();
            }) }
        ] },

      { category: 'Help',
        shortcuts: [
          { keys: ['?'],
            desc: 'show help',
            action: keyHandler(function () {
                if (!popupShowing) {
                    popup.show();
                    popupShowing = true;
                    return false;
                }
            }) },
          { keys: ['esc'],
            desc: 'close help',
            action: function () {
                if (popupShowing) {
                    popup.hide();
                    popupShowing = false;
                    return false;
                }
            } }
        ] }
    ];

    function keyEventWrangler(key) {
        switch (key) {
        case '[':
        case ']':
        case '?':
            return 'keypress';
        default:
            return 'keydown';
        }
    }

    AJS.$("#keyboard-shortcut-help").live('click', function () {
        if (!popupShowing) {
            popup.show();
            popupShowing = true;
            return false;
        }
    });

    // Bind key handlers and generate help html.
    var strs = [];
    AJS.$.each(categories, function (i, category) {
        strs.push('<h3>' + category.category + '</h3>');
        strs.push('<table>');

        AJS.$.each(category.shortcuts, function (j, shortcut) {
            AJS.$.each(shortcut.keys, function (k, key) {
                AJS.$(document).bind(keyEventWrangler(key), { combi: key, disableInInput: true }, shortcut.action);
            });
            strs.push(shortcutHtml(shortcut));
        });

        strs.push('</table>');
    });

    // Create the popup.
    var popupShowing = false;
    var popup = AJS.FECRU.DIALOG.create(800, 550, 'keynavHelp')
        .addHeader('Keyboard Shortcuts')
        .addPanel('All', strs.join(''))
        .addButton('Close', function (dialog) {
            dialog.hide();
            popupShowing = false;
        });

    function shortcutHtml(shortcut) {
        var keysHtml = AJS.$.map(shortcut.keys, keyComboHtml).join(' or ');
        return '<tr><th>' + keysHtml + ' :</th><td>' + shortcut.desc + '</td></tr>';
    }

    function keyComboHtml(keyCombo) {
        return AJS.$.map(keyCombo.split('+'), function (key) {
            return '<kbd>' + key + '</kbd>';
        }).join(' + ');
    }
});
