
function Review() {
    // Member variables
    var m_permaId;
    var m_frxs = {};
    var m_frxCacheDirty = true;
    var m_frxArrayCache = []; // Optimisation array, should never be accessed directly but through this.frxs()
    var m_frxOrderCounter = -1;
    var m_comments = {};
    var m_commentCacheDirty = true;
    var m_commentArrayCache = []; // Optimisation array, should never be accessed directly but through this.comments()
    var m_loaded = false;
    var m_writable = false;
    var m_open = false;
    var m_closed = false;
    var m_draft = false;
    var m_name;
    var m_transitioningState;
    var m_summarize;
    var m_renderTime;

    // Private function which maps an array of [FRX|Comment] -> array of object ids
    var idArray = function(obj) {
        return AJS.$.map(obj, function (obj) {
            return obj.id();
        });
    };

    // Convert to a plain array instead of associative
    var map2array = function(map) {
        var array = [];
        AJS.$.each(map, function() {
            array.push(this);
        });
        array.sort(function(o1, o2) {
            if (o1.order && o2.order) {
                return o1.order() - o2.order();
            } else {
                return 0;
            }
        });

        return array;
    };

    this.id = function() {
        return m_permaId;
    };

    this.setId = function(id) {
        m_permaId = id;
        return this;
    };

    this.isLoaded = function () {
        return m_loaded;
    };

    this.setLoaded = function (loaded) {
        m_loaded = loaded;
        return this;
    };

    this.writable = function() {
        return m_writable;
    };

    this.setWritable = function(writable) {
        m_writable = writable;
        return this;
    };

    this.isOpen = function () {
        return m_open;
    };

    this.setOpen = function (open) {
        m_open = open;
        return this;
    };

    this.isClosed = function () {
        return m_closed;
    };

    this.setClosed = function (closed) {
        m_closed = closed;
        return this;
    };

    this.isDraft = function () {
        return m_draft;
    };

    this.setDraft = function (draft) {
        m_draft = draft;
        return this;
    };

    this.isValid = function() {
        return !!m_name;
    };

    this.isEmpty = function() {
        return this.frxs().length === 0;
    };

    this.setName = function(name) {
        m_name = name;
        return this;
    };

    this.isTransitioning = function() {
        return !!m_transitioningState;
    };

    this.setTransitioningState = function(state) {
        m_transitioningState = state;
        return this;
    };

    this.isSummarize = function() {
        return m_summarize;
    };

    this.setSummarize = function(summarize) {
        m_summarize = summarize;
        return this;
    };

    this.setRenderTime = function (date) {
        m_renderTime = new Date(date.getTime());
        return this;
    };

    this.getRenderTime = function () {
        return new Date(m_renderTime.getTime());
    };

    /******************
     ****   FRXS   ****
     ******************/
    this.frxs = function() {
        if (m_frxCacheDirty) {
            m_frxArrayCache = map2array(m_frxs);
            m_frxCacheDirty = false;
        }
        return m_frxArrayCache;
    };

    this.frxMap = function() {
        return m_frxs;
    };

    this.frx = function(frxId) {
        return m_frxs[frxId];
    };

    this.frxIds = function() {
        return idArray(this.frxs());
    };

    this.addFrx = function(frx) {
        //first remove any existing frx with the same id
        var existingFrx = this.frx(frx.id());
        var order = -1;
        if (existingFrx) {
            order = existingFrx.order();
            this.removeFrx(existingFrx);
        }
        if (order < 0) {
            order = ++m_frxOrderCounter;
        }
        frx.setOrder(order);
        m_frxs[frx.id()] = frx;
        m_frxCacheDirty = true;
        return this;
    };

    this.removeFrx = function(frx) {
        AJS.$.each(frx.comments(), function() {
            review.removeComment(this);
        });
        delete m_frxs[frx.id()];
        m_frxCacheDirty = true;
        return this;
    };

    this.completeFrxs = function() {
        return AJS.$.grep(this.frxs(), function(frx) {
            return frx.isComplete();
        } );
    };

    this.completeFrxIds = function() {
        return idArray(this.completeFrxs());
    };

    this.incompleteFrxs = function() {
        return AJS.$.grep(this.frxs(), function(frx) {
            return !frx.isComplete();
        });
    };

    this.incompleteFrxIds = function() {
        return idArray( this.incompleteFrxs() );
    };

    this.visibleFrxIdsWithComments = function() {
        return idArray(AJS.$.grep(this.frxs(), function(frx) {
            return frx.hasComments() && !frx.isFiltered();
        }));
    };

    this.frxsWithComments = function() {
        return AJS.$.grep(this.frxs(), function(frx) {
            return frx.hasComments();
        });
    };

    this.frxIdsWithComments = function() {
        return idArray(this.frxsWithComments());
    };

    this.frxsWithoutComments = function() {
        return AJS.$.grep(this.frxs(), function(frx) {
            return !frx.hasComments();
        });
    };

    this.frxIdsWithoutComments = function() {
        return idArray(this.frxsWithoutComments());
    };

    this.setFrxsUnloaded = function() {
        AJS.$.each(m_frxs, function() {
            this.setLoaded(false);
        });
        return this;
    };

    /******************
     **** COMMENTS ****
     ******************/
    this.addComment = function(comment) {
        m_comments[comment.id()] = comment;
        m_commentCacheDirty = true;
        return this;
    };

    this.removeComment = function(comment) {
        delete m_comments[comment.id()];
        if(comment.frx()) {
            comment.frx().removeComment(comment);
        }
        m_commentCacheDirty = true;
        return this;
    };

    this.comment = function(id) {
        return m_comments[id];
    };

    this.comments = function() {
        if (m_commentCacheDirty) {
            m_commentArrayCache = map2array(m_comments);
            m_commentCacheDirty = false;
        }
        return m_commentArrayCache;
    };

    this.generalComments = function() {
        return AJS.$.grep( this.comments(), function(comment) {
            return /^general/.test( comment.type() );
        } );
    };

    this.generalCommentIds = function() {
        return idArray( this.generalComments() );
    };

    this.inlineComments = function() {
        return AJS.$.grep( this.comments(), function(comment) {
            return /^inline/.test( comment.type() );
        } );
    };

    this.inlineCommentIds = function() {
        return idArray( this.inlineComments() );
    };

    this.unreadComments = function() {
        return AJS.$.grep( this.comments(), function(comment) {
            return comment.status() === 'unread' || comment.status() === 'leaveUnread';
        } );
    };

    this.draftComments = function () {
        return AJS.$.grep(this.comments(), function (comment) {
            return comment.draft();
        });
    };
}

function Frx(id, review) {
    var m_id = id;
    var m_review = review;
    var m_order = -1;
    var m_isComplete = false;
    var m_colspan = 4;
    var m_isLoaded = false;
    var m_isExpanded = false;
    var m_isFiltered = false;
    var m_isBinary = false;
    var m_isDirectory = false;
    var m_isNewSinceComplete = false;
    var m_renderTime;

    // revision stuff
    var m_frxRevisions = [];
    var m_sliderFrxRevisions;
    var m_visibleFromRevision;
    var m_visibleToRevision;
    var m_visibleFromSCMRevision;
    var m_visibleToSCMRevision;
    var m_comments = {};

    // Methods
    this.id = function() {
        return m_id;
    };

    this.review = function() {
        return m_review;
    };

    this.order = function () {
        return m_order;
    };

    this.setOrder = function(order) {
        m_order = order;
        return this;
    };

    this.isComplete = function() {
        return m_isComplete;
    };

    this.setComplete = function(isComplete) {
        m_isComplete = isComplete;
        return this;
    };

    this.colspan = function() {
        return m_colspan;
    };

    this.setColspan = function(colspan) {
        m_colspan = colspan;
        return this;
    };

    this.isLoaded = function() {
        return m_isLoaded;
    };

    this.setLoaded = function(loaded) {
        m_isLoaded = loaded;
        return this;
    };

    this.isExpanded = function () {
        return m_isExpanded;
    };

    this.setExpanded = function (expanded) {
        m_isExpanded = expanded;
        return this;
    };

    this.isFiltered = function () {
        return m_isFiltered;
    };

    this.setFiltered = function (filtered) {
        m_isFiltered = filtered;
        return this;
    };

    this.isBinary = function() {
        return m_isBinary;
    };

    this.setBinary = function (binary) {
        m_isBinary = binary;
        return this;
    };

    this.isDirectory = function() {
        return m_isDirectory;
    };

    this.setDirectory = function (directory) {
        m_isDirectory = directory;
        return this;
    };

    this.isNewSinceComplete = function() {
        return m_isNewSinceComplete;
    };

    this.setNewSinceComplete = function (newSinceComplete) {
        m_isNewSinceComplete = newSinceComplete;
        return this;
    };

    this.setRenderTime = function (date) {
        m_renderTime = new Date(date.getTime());
        return this;
    };

    this.getRenderTime = function () {
        return new Date(m_renderTime.getTime());
    };

    // FRX Revision Stuff
    this.addFrxRevision = function(frxRev) {
        m_frxRevisions.push(frxRev);
        return this;
    };

    this.frxRevisions = function() {
        return m_frxRevisions;
    };

    this.setSliderFrxRevisions = function (revs) {
        m_sliderFrxRevisions = AJS.$.makeArray(revs);
        return this;
    };

    this.getSliderFrxRevisions = function () {
        return m_sliderFrxRevisions;
    };

    this.setVisibleFromRevision = function(frxRev) {
        m_visibleFromRevision = frxRev;
        return this;
    };

    this.setVisibleToRevision = function(frxRev) {
        m_visibleToRevision = frxRev;
        return this;
    };

    this.setVisibleFromSCMRevision = function (rev) {
        m_visibleFromSCMRevision = rev;
        return this;
    };

    this.setVisibleToSCMRevision = function (rev) {
        m_visibleToSCMRevision = rev;
        return this;
    };

    this.visibleFromRevision = function() {
        return m_visibleFromRevision;
    };

    this.visibleToRevision = function() {
        return m_visibleToRevision;
    };

    this.visibleFromSCMRevision = function() {
        return m_visibleFromSCMRevision;
    };

    this.visibleToSCMRevision = function() {
        return m_visibleToSCMRevision;
    };

    this.hasComments = function() {
        return this.comments().length > 0;
    };

    this.addComment = function(comment) {
        m_comments[comment.id()] = comment;
        return this;
    };

    this.comments = function() {
        var array = [];
        AJS.$.each( m_comments, function() {
            array.push(this);
        } );
        return array;
    };

    this.removeComment = function(comment) {
        delete m_comments[comment.id()];
        return this;
    };
}

function Comment(id, type) {
    var m_id = id;
    var m_frx;
    var m_review;
    var m_replyTo;
    var m_replies = [];
    var m_type = type;
    var m_status;
    var m_message;
    var m_defect = false;
    var m_fromLineRange;
    var m_toLineRange;
    var m_draft = false;
    var m_commentColor;
    var m_gutterLine; // int
    var m_fromRevId;
    var m_toRevId;
    var m_metrics = [];
    var m_issueKey;
    var m_issueStatus;

    // Methods
    this.id = function() {
        return m_id;
    };

    this.domId = function() {
        return m_type + m_id; // if you update this, update visiblePosition()
    };

    // It is possible for the comment object to exist, but not be in the dom - eg when we autosave comments.
    this.domIdExists = function () {
        return AJS.$("#" + this.domId()).length > 0;
    };

    this.frx = function() {
        return m_frx;
    };

    this.setFrx = function(frx) {
        m_frx = frx;
        m_review = frx.review();
        return this;
    };

    this.review = function() {
        return m_review;
    };

    this.setReview = function(review) {
        m_review = review;
        return this;
    };

    this.type = function() {
        return m_type;
    };

    /**
     * the location of the comment: either general, revision or inline.
     */
    this.position = function() {
        return this.isReply() ? m_type.replace(/reply$/, "") : m_type.replace(/comment$/, "");
    };

    /**
     * Returns the position of the comment which is visible.
     * @return either 'general', 'revision', 'inline' or 'above' depending on which comment is visible. If the comment
     * is hidden, an empty string is returned. If both inline and above comments are shown, the result is indeterminant.
     */
    this.visiblePosition = function () {
        var position = this.position();
        var place = this.isReply() ? 'reply' : 'comment';
        if (AJS.$("#" + position + place + m_id).is(":visible")) {
            return position;
        } else if (this.isInline()) {
            position = 'above';
            if (AJS.$('#' + position + place + m_id).is(":visible")) {
                return position;
            }
        }
        return '';
    };

    // todo rename to readStatus
    this.status = function() {
        return m_status;
    };

    this.setStatus = function(status) {
        m_status = status;
        return this;
    };

    this.isReply = function() {
        return !!m_replyTo;
    };

    this.replyTo = function() {
        return m_replyTo;
    };

    this.setReplyTo = function(replyTo) {
        m_replyTo = replyTo;
        return this;
    };

    this.hasReplies = function() {
        return m_replies.length > 0;
    };

    this.addReply = function(comment) {
        m_replies.push(comment);
        return this;
    };

    this.isInline = function() {
        return /^inline/.test(m_type);
    };

    this.isHidden = function () {
        return this.isInline() && !this.fromLineRange() && !this.toLineRange();
    };

    this.message = function() {
        return m_message;
    };

    this.setMessage = function(message) {
        m_message = message;
        return this;
    };

    this.defect = function() {
        return m_defect;
    };

    this.setDefect = function(defect) {
        m_defect = defect;
        return this;
    };

    this.draft = function() {
        return m_draft;
    };

    this.setDraft = function(draft) {
        m_draft = draft;
        return this;
    };

    this.fromLineRange = function() {
        return this.isReply() ? m_replyTo.fromLineRange() : m_fromLineRange;
    };

    this.setFromLineRange = function(lr) {
        m_fromLineRange = lr;
        return this;
    };

    this.toLineRange = function() {
        return this.isReply() ? m_replyTo.toLineRange() : m_toLineRange;
    };

    this.setToLineRange = function(lr) {
        m_toLineRange = lr;
        return this;
    };

    this.fromRevId = function() {
        return this.isReply() ? m_replyTo.fromRevId() : m_fromRevId;
    };

    this.setFromRevId = function(id) {
        m_fromRevId = id;
        return this;
    };

    this.toRevId = function() {
        return this.isReply() ? m_replyTo.toRevId() : m_toRevId;
    };

    this.setToRevId = function(id) {
        m_toRevId = id;
        return this;
    };

    this.commentColor = function() {
        return m_commentColor;
    };

    this.setCommentColor = function(color) {
        m_commentColor = color;
        return this;
    };

    this.gutterLine = function() {
        return this.isReply() ? m_replyTo.gutterLine() : m_gutterLine;
    };

    this.setGutterLine = function(gutterLine) {
        m_gutterLine = gutterLine;
        return this;
    };

    this.metrics = function() {
        return m_metrics;
    };

    this.setMetrics = function(metrics) {
        m_metrics = metrics || [];
        return this;
    };

    this.issueKey = function() {
        return m_issueKey;
    };

    this.setIssueKey = function(issueKey) {
        m_issueKey = issueKey;
        return this;
    };

    this.issueStatus = function() {
        return m_issueStatus;
    };

    this.setIssueStatus = function(issueStatus) {
        m_issueStatus = issueStatus;
        return this;
    };
}

// Both variables are populated in elementIds.jspf
var review = new Review();
var permaId;
