/**
 * Crucible utility functions that can be used from any crucible page.
 */

if (AJS.CRU === undefined) {
    AJS.CRU = {};
}
AJS.CRU.UTIL = {};

(function () {

    /**
     * Crucible base url (without trailing '/').
     *
     * @param permaId optional review id
     */
    AJS.CRU.UTIL.urlBase = function (permaId) {
        if (permaId) {
            return fishEyePageContext + '/cru/' + permaId;
        } else {
            return fishEyePageContext + '/cru';
        }
    };

    /**
     * Crucible JSON base url (without trailing '/').
     *
     * @param permaId optional review id
     */
    AJS.CRU.UTIL.jsonUrlBase = function (permaId) {
        if (permaId) {
            return fishEyePageContext + '/json/cru/' + permaId;
        } else {
            return fishEyePageContext + '/json/cru';
        }
    };
    
    AJS.CRU.UTIL.isReviewPage = function () {
        return typeof review !== 'undefined';
    };


    AJS.CRU.UTIL.startAjaxDialogSpin = function () {
        AJS.$('body').addClass('ajax-dialog');
        AJS.dim();
    };

    AJS.CRU.UTIL.stopAjaxDialogSpin = function () {
        AJS.dim();
        AJS.$('body').removeClass('ajax-dialog');
    };

    AJS.CRU.UTIL.isAjaxDialogSpinning = function () {
        return AJS.CRU.UTIL.isDimmed() && AJS.$('body').hasClass('ajax-dialog');  
    };

    AJS.CRU.UTIL.ajaxDialog = function (url, params) {
        AJS.CRU.UTIL.startAjaxDialogSpin();
        AJS.FECRU.AJAX.ajaxDo(url, params || {}, function (resp) {
            if (resp.worked) {
                if (resp.showDialog) {
                    AJS.CRU.UTIL.stopAjaxDialogSpin();
                    AJS.CRU.DIALOG.$CONTAINER.html(resp.payload);
                } else if (resp.redirect) {
                    // no need to undim
                    window.location = resp.payload;
                }
            }
            // !resp.worked handled by ajaxDo
        });
        return false;
    };

    AJS.CRU.UTIL.stateTransition = function (transition, permaId) {
        var util = AJS.CRU.UTIL;
        var url = util.jsonUrlBase(permaId) + '/changeStateAjax';
        var params = {
            command: transition
        };

        if (util.isReviewPage() && AJS.$.inArray(transition, ['action:completeReview', 'action:summarizeReview']) >= 0) {
            // If completing or summarzing we need to warn if the review has been updated.

            util.startAjaxDialogSpin();
            AJS.CRU.REVIEW.UTIL.reviewUpdatedAjax(function () {
                var reviewUpdated = AJS.$('body').hasClass('review-updated');
                if (reviewUpdated) {
                    AJS.CRU.REVIEW.UTIL.warnAboutReviewUpdates(true);
                }
                util.stopAjaxDialogSpin();
                return util.ajaxDialog(url, AJS.$.extend(params, { reviewUpdated: reviewUpdated }));
            });
            return false;

        } else {
            return util.ajaxDialog(url, params);
        }
    };

    AJS.CRU.UTIL.editDetailsFormChange = false;
    AJS.CRU.UTIL.checkEditForm = function (done) {
        if (AJS.CRU.UTIL.editDetailsFormChange) {
            AJS.CRU.REVIEW.UTIL.postEditDetailsForm(done);
        } else {
            if (done) {
                done();
            }
        }
        return false;//do a link action if called from <a>
    };

    AJS.CRU.UTIL.command = function (cmd, pid, button) {
        var perma = pid || permaId;
        if (button) {
            button.disabled = true;
        }
        var donext = function() {
            var url = AJS.CRU.UTIL.urlBase(perma);
            window.location = url + '/' + cmd;
        };
        //check and post the editDetailsForm if it has changed
        AJS.CRU.UTIL.checkEditForm(donext);
    };


    AJS.CRU.UTIL.createBlankReview = function (params) {
        var url = AJS.CRU.UTIL.jsonUrlBase() + '/createReviewDialog';
        return AJS.CRU.UTIL.ajaxDialog(url, params);
    };

    AJS.CRU.UTIL.addToReview = function (params) {
        var url = AJS.CRU.UTIL.jsonUrlBase() + '/createDialog';
        return AJS.CRU.UTIL.ajaxDialog(url, params);
    };

    AJS.CRU.UTIL.isAnyDialogShowing = function () {
        return AJS.$('body').children('div.dialog:visible').length > 0;
    };

    AJS.CRU.UTIL.isDimmed = function () {
        return !!AJS.dim.dim;
    };

})();
