if (!AJS.FE.CHANGESET) {
    AJS.FE.CHANGESET = {};
}
(function() {
    /////////////////////
    // private functions
    /////////////////////

    /**
     *from http://www.quirksmode.org/js/cookies.html
     */
    function readCookie(name) {
        var nameEQ = name + "=";
        var ca = document.cookie.split(';');
        for (var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) == ' ') {
                c = c.substring(1, c.length);
            }
            if (c.indexOf(nameEQ) == 0) {
                return c.substring(nameEQ.length, c.length);
            }
        }
        return null;
    }

    /*
     * usefull for appending to urls (esp ajax urls) when the content of the
     * url depends on the user's cookie preferences
     */
    function getCookiePrefToken() {
        try {
            var val = readCookie("crucibleprefs1");
            if (val) {
                val = val.replace(/^D=[0-9]+/, "");
                return encodeURIComponent(val);
            }
        } catch(ex) {
            if (AJS.console) {
                AJS.console.log("error getting cookie preference:" +
                            ex.message +
                            "\nin:" + ex.fileName +
                            "\nat:" + ex.lineNumber);
            }
        }
        return "unknown";
    }

    function csDisplaySpinner(revId) {
        var spinnerHTML = "<img src='" + fishEyePageContext + "/" + fishEyeSTATICDIR +
                          "/images/spinner.gif'><span class='loading'>Loading content</span>";
        var divId = csGetCsDivId(revId);
        var divEle = AJS.$("#" + divId);
        divEle.html(spinnerHTML);
        return divId;
    }

    function csGetCsDivId(revId) {
        return 'showfilesDiv' + revId;
    }

    /////////////////////
    // exported functions
    /////////////////////

    AJS.FE.CHANGESET.loadDiff = function (revId, url, onCompleteFunc) {
        var onDone = function() {
            if (onCompleteFunc) {
                onCompleteFunc();
            }
        };
        var $revDiv = AJS.$("#" + revId);
        if ($revDiv.hasClass("unloaded-diff")) {
            var divId = csDisplaySpinner(revId);
            $revDiv.removeClass("unloaded-diff");
            AJS.FECRU.AJAX.ajaxDo(url, {revid: revId, rndtmp: getCookiePrefToken()}, function(resp) {
                if (resp.worked) {
                    // use 'clean' to avoid the regex that jQuery uses to distinguish HTML from ids
                    var replacement = AJS.$.clean([resp.payload], document);
                    AJS.$("#" + divId).html(replacement);
                    if (onCompleteFunc) {
                        onCompleteFunc();
                    }
                }
            }, false);
        } else {
            if (onCompleteFunc) {
                onCompleteFunc();
            }
        }
    };

    /**
     * loads a diff asynchronously.
     * @param i
     * @param url
     * @param revIds
     */
    AJS.FE.CHANGESET.loadDiffAsync = function (i, url, revIds) {
        var revIdLength = revIds.length;
        var onComp = function(origRequest) {
            // do the chaining.
            AJS.FE.CHANGESET.loadDiffAsync(i + 1, url, revIds);
        };
        var MAX_TO_LOAD = 100; //so that if viewing changeset of a branch operation then the page wont overload, set a hard limit.
        if ((i < revIdLength) && (i < MAX_TO_LOAD)) {
            var revId = revIds[i];
            AJS.FE.CHANGESET.loadDiff(revId, url, onComp);
        }
    };

    AJS.FE.CHANGESET.setupChangesetDiffLoading = function() {
        AJS.$("a.unloaded-diff").live("click",
                function(event) {
                    event.preventDefault();
                    var $node = AJS.$(this);
                    var id = $node.parent().parent().attr("id");
                    var baseClUrl = AJS.$("#baseJSONClUrl").val();
                    AJS.FE.CHANGESET.loadDiff(id, baseClUrl, null);
                }
                );
    };

    AJS.FE.CHANGESET.loadDiffForDt = function($dt) {
        var $revIdDiv = $dt.next().children("div.unloaded-diff");
        // if it is already loaded we won't find that div
        if ($revIdDiv) {
            var baseClUrl = AJS.$("#baseJSONClUrl").val();
            AJS.FE.CHANGESET.loadDiff($revIdDiv.attr("id"), baseClUrl, null);
        }
    };

    /**
     * Just disable the link from doing anything.
     */
    AJS.FE.CHANGESET.changesetPathLinkFn = function() {
        return false;
    };

    AJS.FE.CHANGESET.changesetFileLinkFn = function(event) {
        document.location.hash = AJS.$(event.target).children("input").val();
        return false;
    };

})();
