if (!AJS.FECRU) {
    AJS.FECRU = {};
}
AJS.FECRU.AJAX = {};
(function() {
    /******************************************************************
     ******************** AJAX METHODS ********************************
     ******************************************************************/

    AJS.FECRU.AJAX.ajaxDo = function (url, params, onCompleteFunc, noDialog) {
        AJS.FECRU.AJAX.ajaxUpdate(url, params, null, onCompleteFunc, noDialog);
    };

    /**
     * Makes a Ajax request with the specified parameters and runs the onCompleteFunc
     * function when the call returns. Optionally, a DOM element can be passed in
     * which will be updated with the payload of the Ajax response (the JSON response
     * object must contain a "payload" member.
     * Also, the user can pass a second function (onPayloadEvalFunc) that will be
     * called after the payload content has been put in the specified elementToUpdate
     * element. If elementIdToUpdate is null, the onPayloadEvalFunc will not be called.
     *
     * noDialog is a boolean - if true, no error messages will be displayed.
     */
    AJS.FECRU.AJAX.ajaxUpdate = function (url, params, elementIdToUpdate, onCompleteFunc, noDialog) {
        // Implemementation note:
        // jQuery does an eval(req.responseText) for us internally. We have an onSuccess callback to
        // intercept this, otherwise we end up eval'ing the response text twice (which is poor form).
        // The onComplete callback gives us access to the XmlHttpRequest and allows us to do extra
        // error handling if required.

        var data = false;
        var onSuccess = function(response) {
            data = response;
        };

        var onComplete = function(req) {
            var cleanUpOnFail = function () {
                if (onCompleteFunc) {
                    onCompleteFunc({ worked: false });  //allow function to clean up
                }
            };
            try {
                if (req.status == 0 ) {

                    return;
                }
                if (!data || !isECMA(req)) {
                    noDialog || displayUnexpectedResponse(req);
                    cleanUpOnFail();
                    return;
                }

                var $updateMe;
                if (elementIdToUpdate) {
                    $updateMe = AJS.$("#"+elementIdToUpdate);
                }

                if (!data.worked) {
                    AJS.$('#errorDesc').html("<span class='icon-error'>Error</span><p>" + ourHtmlEscape(data.errorMsg));
                    if (!noDialog) {
                        if (data.userError) {
                            showUserErrorBox();
                        } else {
                            showErrorBox();
                        }
                    }
                } else if ($updateMe && $updateMe.length === 1) {
                    $updateMe.html(data.payload);
                }
                if (onCompleteFunc) {
                    onCompleteFunc(data);
                }
            } catch (e) {
                displayErrorInResponse(e);
                cleanUpOnFail()    ;
            }
        };
        AJS.$.ajax( {
            type: "post",
            url: url,
            data: params,
            dataType: "json", // TODO: check that no other callers expect html/xml data
            success: onSuccess,
            complete: onComplete,
            error: noDialog || ajaxFailure
        } );
        return false;
    };

    var ajaxFailure = function (req) {
        try {
            if (!isECMA(req)) {
                displayUnexpectedResponse(req);
                return;
            }
            var resp = eval('(' + req.responseText + ')');
            AJS.$('#errorDesc').html("<span class='icon-error'>Error</span><p>There was an error in the response from the server.<br>" +
                                     "Refresh the page before posting more comments.<br>" +
                                     "Click 'show details' for more information." );
            AJS.$('#showErrorDetails').show();
            appendErrorResponse(resp.errorMsg);
            showErrorBox();
        } catch (e) {
            AJS.log('ajax failure: ' + e);
            displayErrorInResponse(e);
        }
    };

    /******************************************************************
     ******************** HELPER METHODS ******************************
     ******************************************************************/

    var isECMA = function (req) {
        return isContentType(req, /^application\/ecmascript/);
    };

    var isHTML = function (req) {
        return isContentType(req, /^text\/html/);
    };

    var isContentType = function (req, targetType) {
        var respContentType = req.getResponseHeader("Content-Type");
        return respContentType != null && respContentType.match(targetType);
    };

    var isHTMLFrag = function (req) {
        var responseText = req.responseText;
        return responseText != null && !responseText.match(/<body/i);
    };

    var ourHtmlEscape = function (input) {
        try {
            return input.replace(/&/g, '&amp;').
                    replace(/>/g, '&gt;').
                    replace(/</g, '&lt;').
                    replace(/"/g, '&quot;');
        } catch(e) {
            return input.message || e.message;
        }
    };

    AJS.FECRU.AJAX.startSpin = function (id, replace, tagType, className) {
        className = className ? className + " spinner" : "spinner";

        var $el = typeof(id) == 'object' ? AJS.$(id) : AJS.$("#"+id);
        var tag = tagType || 'div';
        if ($el.length === 1) {
            var $spinner = AJS.$(document.createElement(tag));
            $spinner.addClass(className)
                    .html("<img src='" + fishEyePageContext + "/" + fishEyeSTATICDIR + "/2static/images/blank.gif'>");
            if (replace) {
                $el.replaceWith($spinner);
            } else {
                $spinner.insertAfter($el);
            }
            return true;
        }
        return false;
    };

    AJS.FECRU.AJAX.stopSpin = function (el, tagType) {
        var emptyDiv = document.createElement(tagType || "div");
        el.parentNode.replaceChild(emptyDiv, el.nextSibling);
    };

    /******************************************************************
     ******************** ERROR HANDLING ******************************
     ******************************************************************/

    AJS.FECRU.AJAX.checkError = function (resp) {
        if (resp.error) {
            AJS.$('#errorDesc').html('<span class="icon-error">Error</span><p>' + resp.errorHtml);
            return true;
        }
        return false;
    };

    var appendErrorResponse = function (text) {
        if (text) {
            AJS.$('#errorResponses').show();
            AJS.$('#errorResponses').append(AJS.$('<div class="errorResponse-count">Error ' + (AJS.$('#errorResponses').find('.errorResponse').length + 1) + '</div>'));
            AJS.$('#errorResponses').append(AJS.$('<div class="errorResponse"></div>').html( ourHtmlEscape(text) ));
        }
    };

    var dialog = 0;

    var showErrorBox = function () {
        showNotificationBox('An error has occurred');
    };

    var showUserErrorBox = function () {
        showNotificationBox('Cannot perform requested action');
        AJS.$('#showErrorDetails').hide();
        AJS.$('#errorDetails').hide();
    };

    var showNotificationBox = function (title) {
        var util = AJS.CRU.UTIL;
        
        if (dialog == 0) {
            dialog = AJS.FECRU.DIALOG.create(440, 330);
            dialog.addHeader(title)
                .addPanel("Information", AJS.$('#errorBox'))
                .addButton("Reload", function() { window.location.reload(); })
                .addButton("Close", function() {
                    AJS.$('#errorResponses .errorResponse').remove();
                    AJS.$('#errorResponses .errorResponse-count').remove();
                    dialog.hide();
                } );
        }

        if (!util.isAnyDialogShowing()) {
            if (util.isAjaxDialogSpinning()) {
                util.stopAjaxDialogSpin();
            }
            dialog.show();
        }
        return false;
    };

    var displayUnexpectedResponse = function (req) {
        if (isHTML(req)) {
            if (isHTMLFrag(req)) {
                AJS.$('#errorDesc').html("<span class='icon-error'>Error</span><p>There was an error in the response from the server.<br>" +
                                         "Refresh the page before posting more comments.<br>" +
                                         "Click 'show details' for more information." );
                appendErrorResponse(req.responseText);
                showErrorBox();
            } else {
                AJS.$('#errorDesc').html(AJS.$(req.responseText).filter('title').html());
                appendErrorResponse(AJS.$(req.responseText).filter('body').html());
                showErrorBox();
            }
        } else if (!req.responseText) {
            //no-op here: no responseText means most likely server is gone offline.
        }
    };

    var displayErrorInResponse = function (err) {
        AJS.$('#errorDesc').html("<span class='icon-error'>Error</span><p>There was an error in the response from the server.<br>" +
                                 "Refresh the page before posting more comments.<br>" +
                                 "Click 'show details' for more information." );
        appendErrorResponse(err);
        showErrorBox();
    };
})();
