if (!AJS.FECRU) {
    AJS.FECRU = {};
}

(function() {

    /**
     * html for the star edit dialog, to be inserted on demand.
     */
    var DIALOG_HTML = "<div id='astronomy'>" +
                          "<div id='astronomy-scape'>&nbsp;</div>" +
                          "<div id='astronomy-label'>" +
                              "<h4><span>Update favourite</span></h4>" +
                              "<span class='close'><a href='#' class='close-astronomy' id='close-astronomy'>X</a></span>" +
                              "<form action='#'>" +
                                  "<fieldset class='input'>" +
                                      "<label for='star-labels'>Name</label>" +
                                      "<input type='text' id='star-labels' value='Describe your favourite'>" +
                                  "</fieldset>" +
                                  "<fieldset class='button'>" +
                                      "<ul>" +
                                          "<li><button class='remove' id='remove-astronomy'>Remove</button></li>" +
                                          "<li class='odd'><input type='submit' id='save-star-label' value='Save label'></li>" +
                                      "</ul>" +
                                  "</fieldset>" +
                              "</form>" +
                          "</div>" +
                      "</div>";

    ////// private functions

    //for use with the return value of starConstants()
    var ON_ONLY = true;
    var OFF_ONLY = false;
    function starConstants(context) {
        if (!context) {
            context = "body";
        }
        return {
            state: function(on) {
                return on ? { className:"star-on", text:AJS.$("#star-textRemove", context).val() }
                          : { className:"star-off", text:AJS.$("#star-textAdd", context).val() };
            }
        };
    }

    /**
     * todo: see if this can be swapped with the throbber jquery plugin.
     */
    function makeThrobberControl($link, throbberSetting) {
        /**
         * Starts the throbber, which activates after noLatencyThreshold milliseconds have passed.
         * @return a function that will stop the throbber after minThrobberDisplay milliseconds have passed, ensuring no flicker.
         */
        var startThrob = function () {
            $link.data("throbbing", true);
            var timeout = setTimeout(function () {
                $link.addClass("star-throb");
            }, throbberSetting.noLatencyThreshold);

            return function() {
                $link.data("throbbing", false);
                clearTimeout(timeout);
                setTimeout(function () {
                    $link.removeClass("star-throb");
                }, throbberSetting.minThrobberDisplay);
            };
        };

        /**
         * stores a function to stop the throbber for the given $link
         */
        var stopThrob = undefined;
        return {
            start: function() {
                if (throbberSetting.showThrob && !stopThrob) {
                    stopThrob = startThrob();
                }
            },
            stop: function() {
                if (stopThrob) {
                    stopThrob();
                    stopThrob = undefined;
                }
            },
            isThrobbing: function() {
                return $link.data("throbbing");
            }
        };
    }

    /**
     *
     * @param link the jquery element of the link that was clicked
     * @param dialogControl
     * @param options the list of options. Available options are:
     * - throbber : options for throbbing
     *
     */
    function starClicked(link, dialogControl, options) {
        var starId = getStarId(link);
        var starKeys = {};
        var $link = AJS.$(link);
        var throbberControl = makeThrobberControl($link, options.throbberSetting);
        //if throbbing already, then don't do anything.
        if (throbberControl.isThrobbing()) {
            return;
        }

        $link.children("span.inputs").children("input.starKey").each(function() {
            var key = AJS.$(this);
            starKeys[key.attr("name")] = key.val();
        });
        if (starId != null) {
            if (isDialogShown($link)) {
                // we are unstarring, pop up the edit box
                editStar(starId, dialogControl, throbberControl);
            } else {
                //...or remove directly without anymore interaction
                doRemoveStar(starId, throbberControl);
            }
        } else {
            // we are adding a star
            addStar(starKeys, dialogControl, throbberControl);
        }

        // todo: fix this hack to make hoverpopups refresh after starring
        var isUserStar = starKeys["itemType"] === 'atlassian-user';
        var isCommitterStar = starKeys["itemType"] === 'atlassian-committer';
        if (isUserStar || isCommitterStar) {
            //todo: problem is that some committer names displays user hovers, and so the trigger for the hover
            //todo: is no longer available for use here, so cannot work out what the cache key is from the star.
            //todo: refactor this into a better caching mechanism, may be use the DOM to cache instead.
           AJS.FECRU.HOVER.invalidateCache('userlinks');
        }

    }

    function updateAccordian() {
        var $starList = AJS.$("#accordionStarList");
        if ($starList.length > 0) {
            AJS.FECRU.AJAX.ajaxDo(fishEyePageContext + "/json/profile/starAccordionAjaxBody.do", {},
                    function(resp) {
                        if (resp.worked) {
                            $starList.replaceWith(resp.html);
                        }
                    },
                    false
                    );
        }
        var $numStars = AJS.$("#accordionNumStars");
        if ($numStars.length > 0) {
            AJS.FECRU.AJAX.ajaxDo(fishEyePageContext + "/json/profile/starAccordionAjaxNumber.do", {},
                    function(resp) {
                        if (resp.worked) {
                            $numStars.replaceWith(resp.html);
                        }
                    },
                    false
                    );
        }
    }

    function getIdElement(link) {
        return AJS.$(link).children("span.inputs").children("input[name='id']");
    }

    function getStarId(link) {
        return getIdElement(link).val();
    }

    function allStars(on) {
        return AJS.$("a." + starConstants().state(on).className);
    }

    function setStarAttributes(on, $node, context) {
        var constants = starConstants(context);
        var state = constants.state(on);
        var oldState = constants.state(!on);
        $node.removeClass(oldState.className)
             .addClass(state.className)
             .children("span.starText").text(state.text);
    }

    /**
     * Turn on the Star with the given keys
     * @param keys
     */
    function addStar(keys, dialogControl, throbberControl) {
        var paramMap = {};
        for (var key in keys) {
            paramMap["key." + key] = keys[key];
        }
        throbberControl.start();
        doStar(fishEyePageContext + "/json/profile/addStarAjax.do", paramMap, function(newId) {
            allStars(OFF_ONLY).each(function() {
                var $node = AJS.$(this);
                var $inputs = $node.children("span.inputs");
                for (var key in keys) {
                    var input = $inputs.children("input.starKey[name='" + key + "']");
                    if (input.length === 0 || input.val() != keys[key]) {
                        return;
                    }
                }
                setStarAttributes(true, $node, $node);
                $inputs.append("<input type='hidden' name='id' value='" + newId + "'>");
            });
            // AJS.log("id = " + newId);
            if (keys["itemType"] == "atlassian-chart" || keys["itemType"] == "atlassian-search" ||
                keys["itemType"] == "atlassian-quicksearch") {
                editStar(newId, dialogControl, throbberControl);
            }
            throbberControl.stop();
        });
    }

    /**
     * Pop up a dialog box for a Star which is in the on state, allowing the user to set its label or to remove it.
     * @param starId the id of the Star to change.
     * @param dialogControl
     */
    function editStar(starId, dialogControl, throbberControl) {
        // pop up our star edit dialog
        var onSuccess = function(resp) {
            if (resp.worked) {
                var __updateLabel = function(label) {
                    doSaveLabel(starId, label, throbberControl);
                };
                var __removeStar = function() {
                    doRemoveStar(starId, throbberControl);
                };
                showStarDialog(dialogControl, __updateLabel, __removeStar, resp.payload);
            } else {
                //resp.worked is false, reset the star to an off star.
                turnOffStar(starId);
            }
            throbberControl.stop();
        };
        throbberControl.start();
        // get the label for this Star, to supply it to the dialog
        AJS.$.post(fishEyePageContext + "/json/profile/getStarLabelAjax.do", {id: starId}, onSuccess, 'json');
    }

    function turnOffStar(starId) {
        allStars(ON_ONLY).each(function() {
            var id = getStarId(this);
            if (starId == id) {
                // represents the same star
                getIdElement(this).remove();
                var node = AJS.$(this);
                setStarAttributes(false, node, node);
            }
        });
    }

    /**
     * Send a new label value to the server.
     * @param id the id of the Star who's label has been changed
     * @param label a String holding the new label text
     */
    function doSaveLabel(id, label, throbberControl) {
        var onDone = function() {
            refreshDropDown();
            throbberControl.stop();
        };
        throbberControl.start();
        AJS.$.post(fishEyePageContext + "/json/profile/setStarLabelAjax.do", {id:id, label:label}, onDone, 'json');
    }

    /**
     * Unstar the star with the given id.
     *
     * @param id the id of the Star to unstar.
     */
    function doRemoveStar(id, throbberControl) {
        throbberControl.start();
        doStar(fishEyePageContext + "/json/profile/removeStarAjax.do", {id: id}, function() {
            turnOffStar(id);
            throbberControl.stop();
        });
    }

    /**
     * Change the state of a set of stars, and refresh the 'my stars' dropdown.
     * @param url the URL to inform the server about the change
     * @param updateStars the function to call to change the state of the stars in the browser
     */
    function doStar(url, params, updateStars) {
        var done = function(resp) {
            if (resp.worked) {
                updateStars(resp.id);
                refreshDropDown();
                updateAccordian();
                return true;
            }
        };
        AJS.FECRU.AJAX.ajaxDo(url, params, done, false);
    }

    function refreshDropDown() {
        var $dropDown = AJS.$("#starDropDownList");
        if ($dropDown.length > 0) {
            var done = function(resp) {
                if (resp.worked) {
                    $dropDown.replaceWith(resp.html);
                }
            };
            AJS.FECRU.AJAX.ajaxDo(fishEyePageContext + "/json/profile/starDropDownAjax.do", {}, done, false);
        }
    }

    //todo: use AUI inline dialog
    function createStarDialog(dialogControl) {
        var $dialog = AJS.$(DIALOG_HTML);
        //intialize the dialog buttons
        AJS.$("#close-astronomy", $dialog).click(function() {
            dialogControl.hideDialog();
            return false;
        });
        //make the input dialog change style when user start inputting...
        //...and reset to the default if user did not input anything.
        var $starLabel = AJS.$("#star-labels", $dialog);
        $starLabel.focus(function() {
            if ($starLabel.attr("value") === $starLabel.attr("defaultValue")) {
                $starLabel.attr("value", "").addClass("generated");
            }
        }).blur(function() {
            //use the default if input is empty string or space
            if ($starLabel.attr("value").match(/^\s*$/)) {
                $starLabel.attr("value", $starLabel.attr("defaultValue")).removeClass("generated");
            }
        }).keypress(function(evnt) {
            //if user presses enter
            if (evnt.which == 13) {
                AJS.$("#save-star-label", $dialog).trigger('click');
                evnt.preventDefault();
            }
        }).attr("autocomplete", "off");

        return $dialog;
    }

    function showStarDialog(dialogController, updateLabel, removeStar, currentLabel) {
        var labels = currentLabel;
        var $dialog = dialogController.getDialog();
        var $starLabel = AJS.$("#star-labels", $dialog);
        var LABEL_TIP = $starLabel.attr("defaultValue");

        labels = (labels !== "") ? labels : LABEL_TIP;

        $starLabel.attr("value", labels);
        if ($starLabel.attr("value") !== LABEL_TIP) {
            $starLabel.addClass("generated");
        }
        else {
            $starLabel.removeClass("generated");
        }
        AJS.$("#remove-astronomy", $dialog).unbind().click(function(evnt) {
            evnt.preventDefault();
            removeStar();
            dialogController.hideDialog();
        });

        AJS.$("#save-star-label", $dialog).unbind().click(function(evnt) {
            evnt.preventDefault();
            var newLabel = $starLabel.val();
            if (newLabel !== LABEL_TIP) {
                updateLabel(newLabel);
            }
            dialogController.hideDialog();
        });
        dialogController.showDialog();
    }

    function isDialogShown(starLink) {
        return starLink.hasClass("showDialog");
    }

    /**
     * bind a live click event to the stars.
     */
    function bindStars() {
        var $dialog = undefined; //stores a reference to the jquery elmem
        var dialogHider = {};
        var PADDING = 5;
        var hoverOptions = {
            onHover: false,
            showArrow: true,
            fadeTime: 200,
            hideDelay: 200,
            showDelay: 0,
            width: 240 + PADDING,
            offsetX: -8, //to get the arrow centred at the star's vertical asix
            offsetY: 0,
            container: "body",
            useLiveEvents: true,
            cacheContent:false,
            initCallback: function() {
                //implementation note: we need a way to hide the dialog on demand - this callback has a hook into
                //the hide function that inline-dialog uses internally, so this is saving a reference to that function
                //for use later.
                var that = this;
                dialogHider.hideDialog = function() {
                    that.hide();
                };
            }
        };
        var onStarClickedHandler = function ($contentDiv, trigger, showPopup) {
            var $link = AJS.$(trigger);
            var dialogControl = {
                showDialog: function() {
                    if (isDialogShown($link)) {
                        if ($dialog === undefined) {
                            $dialog = createStarDialog(dialogHider);
                        }
                        $dialog.appendTo($contentDiv).show();
                        showPopup();
                    }
                },
                hideDialog: function() {
                    dialogHider.hideDialog();
                },
                getDialog: function () {
                    if ($dialog === undefined) {
                        $dialog = createStarDialog(dialogHider);
                    }
                    return $dialog;
                }
            };
            var opts = {
                throbberSetting: {
                    showThrob: true,         //todo: customizable throbbing needed?
                    noLatencyThreshold: 150, //if the ajax call takes less than this milliseconds, then no throb shown
                    minThrobberDisplay: 200  //otherwise, show for at least this milliseconds.
                }
            };
            starClicked(trigger, dialogControl, opts);
        };
        AJS.InlineDialog(".starrable", "star-inline-dialog", onStarClickedHandler, hoverOptions);
    }

    ////// onload events for stars
    AJS.toInit(function() {
        bindStars();
    });
})();
