// class ActivityFeed
function ActivityFeed(feedId, baseUrl, resourceUrl, feedUrl)
{
    this.feedId = feedId;
    this.baseUrl = baseUrl;
    this.resourceUrl = resourceUrl;
    this.feedUrl = feedUrl;
    
    this.aggregatedItemsCount = 0;
    
	this.maxResults = 20;
	this.keys = "";
	this.filterUsers = "";
	this.itemKeys = "";
	this.filterKeys = "";

    if (feedUrl)
    {
	    var urlSplits = feedUrl.split("?");
	    if (urlSplits.length > 1)
	    {
	    	var feedParams = urlSplits[1].split("&");
	    	for (var i = 0; i < feedParams.length; i++)
	    	{
	    		var paramNameValue = feedParams[i].split("=");
	    		if (paramNameValue[0] == 'maxResults' && (parseInt(paramNameValue[1]) > 0))
	    			this.maxResults = parseInt(paramNameValue[1]);
	    		else if (paramNameValue[0] == 'key')
	    			this.keys += paramNameValue[1] + ' ';
	    		else if (paramNameValue[0] == 'filterUser')
	    			this.filterUsers += paramNameValue[1] + ' ';
	    		else if (paramNameValue[0] == 'itemKey')
	    			this.itemKeys += paramNameValue[1] + ' ';
	    		else if (paramNameValue[0] == 'filter')
	    			this.filterKeys += paramNameValue[1] + ' ';
	    	}
	    }
    }
    
    this.fetchSize = this.maxResults;
    
    YAHOO.util.Event.addListener("showMoreLink-" + feedId, "click", this.showMoreResults, this, true);
    this.COMMENT_ICON_HTML = '<img class="activity-item-comment-icon" style="visibility:hidden" src="'
            + resourceUrl + '/showcomment"/>';
}

ActivityFeed.prototype.setFeedUrl = function(feedUrl)
{
    this.feedUrl = feedUrl;
};

ActivityFeed.prototype.toggleShowMoreLink = function(show)
{
    this.setMoreAndWaitingLinkVisibility(show, !show);
};

ActivityFeed.prototype.hideShowMoreAndWaitingLinks = function()
{
    this.setMoreAndWaitingLinkVisibility(false, false);
};

ActivityFeed.prototype.setMoreAndWaitingLinkVisibility = function(showMoreLink, showWaitingLink)
{
    var moreLink = document.getElementById("showMoreLink-" + this.feedId);
    if (moreLink)
    {
        moreLink.style.display = showMoreLink ? 'inline' : 'none';
    }
    var moreWaiting = document.getElementById("moreWaiting-" + this.feedId);
    if (moreWaiting)
    {
        moreWaiting.style.display = showWaitingLink ? 'inline' : 'none';
    }
};

ActivityFeed.prototype.showMoreResults = function()
{
    this.toggleShowMoreLink(false);
    this.maxResults += this.fetchSize;
    this.aggregatedItemsCount = 0;
    this.populateFeed();
};

ActivityFeed.prototype.addItem = function(feedItem, dateFormat)
{
    var dateContainer = this.getOrCreateDateContainer(DateUtils.format(DateUtils.utcStringToDateByHour(feedItem.updated), dateFormat));
    var items = dateContainer.lastChild;
    if (items.tagName !== 'UL')
    {
        this.setError('Found unexpected element ' + items.tagName + ' in ' + dateContainer.id);
        return;
    }
    var item = document.createElement('li');
    item.className = "activity-item";
    if (feedItem.icon)
    {
        item.style.background = "no-repeat left top url('" + feedItem.icon + "')";
    }
    item.style.paddingLeft = "24px";
    item.style.verticalAlign = "top";
    item.innerHTML += feedItem.summary;
    items.appendChild(item);
};

ActivityFeed.prototype.getOrCreateDateContainer = function(date)
{
    var containerId = 'dateContainer-' + this.feedId + '-' + date;
    var container = document.getElementById(containerId);
    if (container == null)
    {
        var div = document.getElementById('feedContainer-' + this.feedId);

        container = document.createElement('div');
        container.id = containerId;
        div.appendChild(container);

        var dateLabel = document.createElement('h3');
        YAHOO.util.Dom.addClass(dateLabel, "timestamp");
        dateLabel.innerHTML = date;
        container.appendChild(dateLabel);
        var items = document.createElement('ul');
        YAHOO.util.Dom.addClass(items, 'activityItems');
        container.appendChild(items);
    }

    return container;
};

ActivityFeed.prototype.getDateFormat = function(minDate, maxDate)
{
    if (minDate == null || maxDate == null)
        return "MMMMM dd - hh aa";

    var delta = maxDate.getTime() - minDate.getTime();

    if (delta <= DateUtils.MILLIS_PER_DAY * 2)
        return "MMMMM dd - hh aa";
    else if (delta <= DateUtils.MILLIS_PER_DAY * 30)
        return "MMMMM dd";
    else
        return "MMMMM yyyy";
};

/**
 * Aggregates the feed.
 * @param feed
 */
ActivityFeed.prototype.aggregateFeed = function(feed)
{
	var aggregatedItemsCount = 0;
    var newItems = new Array();
    for (var i = 0; i < feed.items.length; i++)
    {
        var item = feed.items[i];
        // Find a matching item
        var matchingItem = null;
        if (item.action && item.inReplyTo && item.category)
        {
            for (var j = 0; j < newItems.length; j++)
            {
                var tmpItem = newItems[j];
                if (item.inReplyTo == tmpItem.inReplyTo && item.category == tmpItem.category)
                {
                    matchingItem = tmpItem;
                    break;
                }
            }
        }
        if (matchingItem)
        {
        	aggregatedItemsCount++;
            if (!matchingItem.multiple)
            {
                matchingItem.multiple = true;
                // Prepend description with the usersname and wrap in li
                matchingItem.description = this.getMultipleDescription(matchingItem);
            }
            // Add the authors if they don't already exist
            for (var k = 0; k < item.authors.length; k++)
            {
                var found = false;
                for (var l = 0; l < matchingItem.authors.length; l++)
                {
                    if (item.authors[k].name == matchingItem.authors[l].name)
                    {
                        found = true;
                        break;
                    }
                }
                if (!found)
                {
                    matchingItem.authors.push(item.authors[k]);
                }
            }
            // Add the description
            matchingItem.description += this.getMultipleDescription(item);
        }
        else
        {
            // Just add
            newItems.push(item);
        }
    }
    feed.items = newItems;
    return aggregatedItemsCount;
};

/**
 * Gets the description for a activity item that's being combined with other activity items during aggregation.  Will
 * add comment icons and forms to the description if necessary.
 *
 * @param item
 */
ActivityFeed.prototype.getMultipleDescription = function(item)
{
    if (item.description)
    {
        var description = '<li class="activity-item-description">';
        if (item.replyTo)
        {
            description += this.COMMENT_ICON_HTML;
        }
        description += '<div class="activity-item-description-author">' +
                       item.authors[0].name + ':</div> ' + item.description;
        if (item.commentForm)
        {
            description += item.commentForm;
        }
        description += '</li>';
        return description;
    }
};

/**
 * Updates the feed summaries.  This should be done after aggregation, and is responsible for rendering the
 * list of authors, comment icons/forms in the right places, and adding comment behaviour.
 * @param feed The feed to update the summaries of.
 */
ActivityFeed.prototype.updateFeedSummaries = function(feed)
{
    for (var i = 0; i < feed.items.length; i++)
    {
        var item = feed.items[i];
        if (item.action)
        {
            // Update the summary
            var summary = "";
            // If there's no description but there is a comment form, the icon needs to be added to the
            // action line before the authors
            if (!item.description && item.replyTo)
            {
                summary += this.COMMENT_ICON_HTML;
            }
            // Render the list of authors.
            var authorCount = item.authors.length;
            for (var m = 0; m < authorCount; m++)
            {
                if (item.authors[m].uri)
                {
                    summary +=
                    '<a class="activity-item-author" href="' + item.authors[m].uri + '">' + item.authors[m].name +
                    '</a>';
                }
                else
                {
                    summary += '<span class="activity-item-author">' + item.authors[m].name + '</span>';
                }
                var authorsLeft = authorCount - m - 1;
                if (authorsLeft == 0)
                {
                    summary += " ";
                }
                else if (authorsLeft == 1)
                {
                    summary += " and ";
                }
                else
                {
                    summary += ", ";
                }
            }
            summary += item.action;

            // Add the description, if it exists
            if (item.description)
            {
                // If there's a linking text between the action and the description, add it.
                if (item.linkingText)
                {
                    summary += " " + item.linkingText;
                }
                summary += ":";
                if (item.multiple)
                {
                    summary += ' <ul class="activity-item-descriptions">' + item.description + '</ul>';
                }
                else
                {
                    summary += ' <div class="activity-item-description">';
                    // If it's not multiple, the comment forms and icon are yet to be added.
                    if (item.commentForm)
                    {
                        summary += this.COMMENT_ICON_HTML + item.description + item.commentForm;
                    }
                    else
                    {
                        summary += item.description;
                    }
                    summary += '</div>';
                }
            }
            else if (item.commentForm)
            {
                // If there's no description, but there is a comment form, the comment form needs to be added
                // to the end of the main summary.
                summary += item.commentForm;
            }
            item.summary = summary;
        }
    }
};

/**
 * Adds comment forms to all feeds that have a replyTo set.  This should be called before aggregation,
 * because the aggregation depends on it.
 * @param feed The feed to add comment forms to.
 */
ActivityFeed.prototype.addCommentForms = function(feed)
{
    for (var i = 0; i < feed.items.length; i++)
    {
        var item = feed.items[i];
        if (item.replyTo)
        {
            item.commentForm = '<form method="POST" class="activity-item-comment-form">' +
                               '<input type="hidden" name="replyTo" value="' + item.replyTo + '"/>' +
                               '<textarea name="comment" rows="6" cols="40"></textarea>' +
                               '<button name="submit">Save</button></form>';
        }
    }
};

ActivityFeed.prototype.populateFeed = function()
{
    var showCommentButton = this.resourceUrl + "/showcomment",
            hideCommentButton = this.resourceUrl + "/hidecomment";

    var thisFeed = this;

    var handleSuccess = function(o)
    {
        thisFeed.setDivText('feedContainer-' + thisFeed.feedId, ''); // remove the loading graphic

        if (o.responseText !== undefined)
        {
            var feed = new JAtom(o.responseXML);
            if (!feed || !feed.items || feed.items.length == 0)
            {
                thisFeed.setWarning('No activity was found');
                return;
            }

            thisFeed.setError(null);
            thisFeed.setWarning(null);
            thisFeed.addCommentForms(feed);
            var aggregatedItemsCount = thisFeed.aggregateFeed(feed);
            if (aggregatedItemsCount > thisFeed.aggregatedItemsCount)
            {
            	thisFeed.aggregatedItemsCount = aggregatedItemsCount;
            	if (feed.items.length + thisFeed.aggregatedItemsCount >= thisFeed.maxResults)
            	{
            		thisFeed.populateFeed();
            		return;
            	}
            }
            thisFeed.updateFeedSummaries(feed);

            var maxDate, minDate;
            for (var i = 0; i < feed.items.length; i++)
            {
                var timestamp = DateUtils.utcStringToDateByHour(feed.items[i].updated);
                if (i == 0)
                {
                    maxDate = timestamp;
                    minDate = timestamp;
                }
                else if (timestamp.getTime() > maxDate.getTime())
                {
                    maxDate = timestamp;
                }
                else if (timestamp.getTime() < minDate.getTime())
                    {
                        minDate = timestamp;
                    }
            }

            var dateFormat = thisFeed.getDateFormat(minDate, maxDate);

            for (i = 0; i < feed.items.length; i++)
            {
                thisFeed.addItem(feed.items[i], dateFormat);
            }

            // STRM-124: if we asked for 10 results and got back 5, there's no point displaying the showMore link
            if (feed.items.length < thisFeed.maxResults)
            {
                thisFeed.hideShowMoreAndWaitingLinks();
            }
            else
            {
                thisFeed.toggleShowMoreLink(true);
            }
        }
        else
        {
            thisFeed.toggleShowMoreLink(true);
        }

        var jqueryHoverFunction = function()
        {
            jQuery(this).hover(function ()
            {
                // Would love to just use fadeIn fadeOut here, but we can't, because they use
                // display: None at the end, but we want the icon to still take up space
                jQuery(this).children("img.activity-item-comment-icon").css("visibility", "visible").fadeIn("fast");
                jQuery(this).css("background-color", "#FFFFF0");
            }, function()
            {
                jQuery(this).children("img.activity-item-comment-icon").fadeOut("fast", function()
                {
                    jQuery(this).css("visibility", "hidden").css("display", "inline");
                });
                jQuery(this).css("background-color", "transparent");
            });
        };
        // Add comment behaviour
        jQuery(".activity-item-description:has(.activity-item-comment-icon)").each(jqueryHoverFunction);
        jQuery(".activity-item:has(.activity-item-comment-icon):not(:has(.activity-item-description))").each(jqueryHoverFunction);

        jQuery(".activity-item-comment-icon")
                .click(function ()
        {
            jQuery(this).parent().children("form.activity-item-comment-form").slideToggle();
        })
                .toggle(function ()
        {
            jQuery(this).attr("src", hideCommentButton);
        }, function()
        {
            jQuery(this).attr("src", showCommentButton);
        });

        jQuery(".activity-item-comment-form")
                .children()
                .focus(function()
        {
            jQuery(this).parent().children("button").attr("accesskey", "s");
        })
                .blur(function()
        {
            jQuery(this).parent().children("button").removeAttr("accesskey");
        })
                .end()
                .submit(function(event)
        {
            event.preventDefault();
            var form = jQuery(this);

            /* todo make go through the issueRequest abstraction so it will work with gadgets API
            thisFeed.issueRequest("POST", thisFeed.baseUrl + "/plugins/servlet/streamscomments", "comment="+escape(this["comment"]
                    +"&replyTo=" + escape(this["replyTo"])), function()
            {
                form.children("textarea").val("").end();
                form.prev(".activity-item-comment-icon").click();
                thisFeed.populateFeed();
            }, null);
            */
            jQuery.post(thisFeed.baseUrl + "/plugins/servlet/streamscomments", jQuery(this).serialize(), function()
            {
                form.children("textarea").val("").end();
                form.prev(".activity-item-comment-icon").click();
                thisFeed.populateFeed();
            }, "text");
        });
        thisfeed.feedPopulated();
    };

    var handleFailure = function(o)
    {
        thisFeed.setDivText('feedContainer-' + thisFeed.feedId, ''); // remove the loading graphic
        thisFeed.setError('Error communicating with server: ' + o.statusText);
        thisFeed.toggleShowMoreLink(true);
    };

    var sUrl = this.feedUrl.split('?')[0];
    var paramString = "";
    if (!isNaN(this.maxResults) && parseInt(this.maxResults) > 0)
    	paramString += '&' + "maxResults=" + (this.maxResults + this.aggregatedItemsCount);
    paramString = appendUrlParams(paramString, "key", this.keys);
    paramString = appendUrlParams(paramString, "filterUser", this.filterUsers);
    paramString = appendUrlParams(paramString, "itemKey", this.itemKeys);
    paramString = appendUrlParams(paramString, "filter", this.filterKeys);

    this.issueRequest("GET", sUrl + '?' + paramString.substring(1), null, handleSuccess, handleFailure);

};

function appendUrlParams(paramStr, paramName, paramValues)
{
    if (trimString(paramValues).length > 0)
    {
    	var values = paramValues.split(' ')
    	for (var i = 0; i < values.length; i++)
    	{
    		if (trimString(values[i]).length > 0)
    			paramStr += '&' + paramName + '=' + trimString(values[i]);
    	}
    }
    return paramStr;
}

function trimString(str)
{
	return str.replace(/^\s+|\s+$/g, '');
}

ActivityFeed.prototype.issueRequest = function (method, url, data, successCallback, failureCallback)
{
    var callback =
    {
        success: successCallback,
        failure: failureCallback
    };

    YAHOO.util.Connect.asyncRequest(method, url, callback);
};

ActivityFeed.prototype.feedPopulated = function()
{
    
};

ActivityFeed.prototype.setWarning = function(warning)
{
    this.setDivText('feedWarning-' + this.feedId, warning);
};

ActivityFeed.prototype.setError = function(error)
{
    this.setDivText('feedError-' + this.feedId, error);
};

ActivityFeed.prototype.setDivText = function(divId, text)
{
    var div = document.getElementById(divId);
    if (div)
    {
        // The inner div is so that we can add padding in the CSS without having the error box appear when empty
        div.innerHTML = text ? '<div class="inner">' + text + '</div>' : '';
    }
    else
    {
        alert(text);
    }
};

ActivityFeed.toggleStackTraces = function()
{
    jQuery(".activityFeed .error .stack-trace").toggle();
};

// end class ActivityFeed

function isNumeric(sText)
{
    var ValidChars = "0123456789";
    var Char;

    for (var i = 0; i < sText.length; i++)
    {
        Char = sText.charAt(i);
        if (ValidChars.indexOf(Char) == -1)
        {
            return false;
        }
    }
    return true;
}

// config panel
YAHOO.namespace("jira.stream.config");

function initConfigPanel(activityFeed)
{
    var elementAlign = "tr";
    var contextAlign = "bl";
    var width = 200;

    var feedId = activityFeed.feedId;
    YAHOO.jira.stream.config.panel = new YAHOO.widget.Panel("panel2-" + feedId,
    { visible : false,
        width : width + "px",
        draggable:false,
        close:true,
        context : ["configure-icon-" + feedId, elementAlign, contextAlign]
    });

    // set focus on the input field and select it when the dialog is displayed
    YAHOO.jira.stream.config.panel.showEvent.subscribe(function()
    {
        document.getElementById('maxResults-' + feedId).focus();
        document.getElementById('maxResults-' + feedId).select();
    });

    YAHOO.jira.stream.config.panel.hideEvent.subscribe(function(activityFeed)
    {
        return function()
        {
        	var populate = false;
            var value = document.getElementById("maxResults-" + activityFeed.feedId).value;
            if (value != activityFeed.maxResults && isNumeric(value))
            {
            	activityFeed.maxResults = parseInt(value);
            	activityFeed.fetchSize = activityFeed.maxResults;
            	populate = true;
            }
            else
            {
            	document.getElementById("maxResults-" + activityFeed.feedId).value = activityFeed.maxResults;
            }
            value = document.getElementById("keys-" + activityFeed.feedId).value;
            if (value != activityFeed.keys)
            {
            	activityFeed.keys = value;
            	populate = true;
            }
            value = document.getElementById("filterUsers-" + activityFeed.feedId).value;
            if (value != activityFeed.filterUsers)
            {
            	activityFeed.filterUsers = value;
            	populate = true;
            }
            value = document.getElementById("itemKeys-" + activityFeed.feedId).value;
            if (value != activityFeed.itemKeys)
            {
            	activityFeed.itemKeys = value;
            	populate = true;
            }
            value = document.getElementById("filterKeys-" + activityFeed.feedId).value;
            if (value != activityFeed.filterKeys)
            {
            	activityFeed.filterKeys = value;
            	populate = true;
            }
            if (populate)
            {
            	activityFeed.aggregatedItemsCount = 0;
            	activityFeed.populateFeed();
            }
        };
    }(activityFeed));

    // create a custom show function so that it will re-align based on the dimensions of the current page... anything
    // that pushes its place on the page down further such as activity feed items being added will mess up its placement
    // in relation to its context object
    var showPanelFunction = function()
    {
        YAHOO.jira.stream.config.panel.align(elementAlign, contextAlign);
        YAHOO.jira.stream.config.panel.show();
    };

    var holder = document.createElement('span');

    var div = document.createElement('div');
    div.id = 'container-' + feedId;
    holder.appendChild(div);

    var form = document.createElement('form');
    form.id = 'feed-form-' + feedId;
    div.appendChild(form);
    
    createInputField(form, 'Max items', "maxResults-" + feedId, 'text', activityFeed.maxResults, 4, 'numeric');
    createInputField(form, 'Project keys', "keys-" + feedId, 'text', activityFeed.keys, 10, null);
    createInputField(form, 'Users', "filterUsers-" + feedId, 'text', activityFeed.filterUsers, 10, null);
    createInputField(form, 'Search these words', "itemKeys-" + feedId, 'text', activityFeed.itemKeys, 10, null);
    createInputField(form, 'Filter', "filterKeys-" + feedId, 'text', activityFeed.filterKeys, 10, null);

    //YAHOO.jira.stream.config.panel.setHeader("<b class='spiffy'><b class='spiffy1'><b></b></b><b class='spiffy2'><b></b></b><b class='spiffy3'></b><b class='spiffy4'></b><b class='spiffy5'></b></b>");
    YAHOO.jira.stream.config.panel.setBody(holder.innerHTML);
    //YAHOO.jira.stream.config.panel.setFooter("<b class='spiffy'><b class='spiffy5'></b><b class='spiffy4'></b><b class='spiffy3'></b><b class='spiffy2'><b></b></b><b class='spiffy1'><b></b></b></b>");

    // Render the Dialog
    YAHOO.jira.stream.config.panel.render();
    // this needs to be set to prevent the span from taking the entire width and thus screwing up its relative positioning
    YAHOO.util.Dom.setStyle('panel2-' + feedId + "_c", 'width', width + "px");

    YAHOO.util.Event.addListener('feed-form-' + feedId, 'submit',
            function(e)
            {
                YAHOO.util.Event.preventDefault(e);
                YAHOO.jira.stream.config.panel.hide();
            });

    var keyListener = new YAHOO.util.KeyListener(document, {keys: [27, 13]},
    {fn:YAHOO.jira.stream.config.panel.hide,
        scope:YAHOO.jira.stream.config.panel,
        correctScope:true});
    keyListener.enable();

    // add an event handler for clicks on the configuration icon to show this dialog
    YAHOO.util.Event.addListener("configure-icon-" +
                                 feedId, "click", showPanelFunction, YAHOO.jira.stream.config.panel, true);
}

function createInputField(form, labelHtml, inputId, inputType, inputValue, inputSize, inputClass)
{
    var label = document.createElement('div');
    label.innerHTML = labelHtml;
    YAHOO.util.Dom.addClass(label, 'label');
    form.appendChild(label);
    var input = document.createElement('input');
    input.id = inputId;
    input.type = inputType;
    input.size = inputSize;
    input.defaultValue = inputValue;
    if (inputClass)
    	YAHOO.util.Dom.addClass(input, inputClass)
    form.appendChild(input);
}

var DateUtils =
{
    /**
     * takes a string and formats it based on the rules of Java's simple data format
     * yyyy - 4 digit year
     * yy - 2 digit year
     * MM - 2 digit month
     * MMM - 3 letter month abbrev
     * MMMMM - month word
     * d - day in month
     * k - hour in day (1-24)
     * h - hour in AM/PM
     * a - am/pm marker
     * m - minute in hour
     * s - second in minute
     *
     * @param date
     * @param formatString
     */
    format:function (date, formatString)
    {
        var str = formatString.replace("yyyy", date.getFullYear(), "g");
        str = str.replace("yy", DateUtils.getShortYear(date), "g");
        str = str.replace("MMMMM", DateUtils.getLongMonth(date), "g");
        str = str.replace("MMM", DateUtils.getMonthAbbrev(date), "g");
        str = str.replace("MM", date.getMonth() + 1, "g");
        str = str.replace("dd", date.getDate(), "g");
        str = str.replace("kk", date.getHours() + 1, "g");
        str = str.replace("hh", DateUtils.getHourIn12(date), "g");
        str = str.replace("aa", DateUtils.getAmOrPm(date), "g");
        str = str.replace("mm", date.getMinutes(), "g");
        str = str.replace("ss", date.getSeconds(), "g");

        return str;
    },

    getShortYear:function (date)
    {
        var year = date.getYear() % 100;
        return year < 10 ? '0' + year : year;
    },

    getLongMonth:function (date)
    {
        return ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
                [date.getMonth()];
    },

    getMonthAbbrev:function (date)
    {
        return ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][date.getMonth()];
    },

    getHourIn12:function(date)
    {
        if (date.getHours() == 0)
            return 12;
        else
            return date.getHours() > 12 ? date.getHours() - 12 : date.getHours();
    },

    getAmOrPm:function(date)
    {
        return date.getHours() < 12 ? "am" : "pm";
    },

    /** useful for ROME formatted timestamps in feeds */
    utcStringToDateByHour: function(string)
    {
        var date = new Date();

        date.setUTCFullYear(string.substr(0, 4));
        date.setUTCMonth(string.substr(5, 2) - 1);
        date.setUTCDate(string.substr(8, 2));
        date.setUTCHours(string.substr(11, 2));
        date.setUTCMinutes(0);
        date.setUTCSeconds(0);
        date.setUTCMilliseconds(0);

        return date;
    }
};

DateUtils.MILLIS_PER_DAY = 1000 * 60 * 60 * 24;
